//
//  KColorSyntaxServer.m
//  KEdit
//
//  Created by Karino, Masatsugu on Mon May 26 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

/*
	If you attempt to add new mode, modify these methods;
		'adjustColorSyntaxWithRange:andFont:', 
		'resetSettings', 
		'setTextType:'.
	And add an internal method for treating colors.
*/

#import "KColorSyntaxServer.h"
#import "KSettingFile.h"
#import "KStringUtils.h"
#import "KRangeObj.h"
#import "KPrefsServer.h"
#import "KApplicationDel.h"

@implementation KColorSyntaxServer

- (id)initWithTextStorage:(NSTextStorage*)storage ofType:(KDocumentTextFileFormatType)type
{
	[super init];
	mSettings = nil;
	mKeywords = nil;
	mPrevCommentCount = 0;
	
	[self setTextType:type];
	[self setTextStorage:storage];
		
	return self;
}

+ (id)colorSyntaxServerWithTextStorage:(NSTextStorage*)storage ofType:(KDocumentTextFileFormatType)type
{
	return [[[self alloc] initWithTextStorage:storage ofType:type] autorelease];
}

- (void)adjustColorSyntaxWithRange:(NSRange)validatedRange andFont:(NSFont*)font
{
	[mTextStorage beginEditing];
	
	// for testing.
	//validatedRange = NSMakeRange(0, [mTextStorage length]);
	
	[mTextStorage addAttribute:NSFontAttributeName value:font range:validatedRange];
	
	switch((int)mTextType){
		case KDocumentTextFileFormat_Plain: [self validateColorSyntaxPlain:validatedRange]; break;
		case KDocumentTextFileFormat_HTML: [self validateColorSyntaxHTML:validatedRange]; break;
		case KDocumentTextFileFormat_CSS: [self validateColorSyntaxCSS:validatedRange]; break;
		case KDocumentTextFileFormat_KIndex: [self validateColorSyntaxKMarkup:validatedRange]; break;
		case KDocumentTextFileFormat_PHP: [self validateColorSyntaxPHP:validatedRange]; break;
		case KDocumentTextFileFormat_Perl: [self validateColorSyntaxPerl:validatedRange]; break;
		case KDocumentTextFileFormat_Ruby: [self validateColorSyntaxRuby:validatedRange]; break;
		case KDocumentTextFileFormat_Indexed: [self validateColorSyntaxIndexed:validatedRange]; break;
	}
	
	[mTextStorage endEditing];
}

// accessor methods.
- (KDocumentTextFileFormatType)textType 
{
	return mTextType;
}

- (void)setTextType:(KDocumentTextFileFormatType)type
{
	if(type == KDocumentTextFileFormat_Plain 
			|| type == KDocumentTextFileFormat_HTML 
			|| type == KDocumentTextFileFormat_Indexed 
			|| type == KDocumentTextFileFormat_KIndex
			|| type == KDocumentTextFileFormat_CSS 
			|| type == KDocumentTextFileFormat_Perl 
			|| type == KDocumentTextFileFormat_PHP
			|| type == KDocumentTextFileFormat_Ruby){
		mTextType = type;
		[self resetSettings];
	}else{
		NSLog(@"KColorSyntaxServer::setTextType - non-supported type.");
	}
	
}

- (NSTextStorage*)textStorage { return mTextStorage; }
- (void)setTextStorage:(NSTextStorage*)storage { mTextStorage = storage; }
- (KModeSettings*)settings { return mSettings; }

// internal use.
- (void)resetSettings
{
	[mSettings release];
	[mKeywords release];
	mSettings = nil;
	mKeywords = nil;
	mPrevCommentCount = 0;
	
	switch((int)mTextType){
		case KDocumentTextFileFormat_Plain:{
      mSettings = [[KModeSettings alloc] initWithFileOfName:KModeSettingFileName_Plain ofType:@"txt"];
      break;
    }
		case KDocumentTextFileFormat_HTML:{
			mSettings = [[KModeSettings alloc] initWithFileOfName:KModeSettingFileName_HTML ofType:@"txt"];
			break;
		}
		case KDocumentTextFileFormat_Indexed:{
			mSettings = [[KModeSettings alloc] initWithFileOfName:KModeSettingsFileName_IndexedText ofType:@"txt"];
			mKeywords = [[KKeywords alloc] initWithFileOfName:KKeywordsFileName_IndexedText ofType:@"txt" withSeparators:@", \t\r\n"];
			break;
		}
		case KDocumentTextFileFormat_KIndex:{
			mSettings = [[KModeSettings alloc] initWithFileOfName:KModeSettingsFileName_KIndex ofType:@"txt"];
			break;
		}
		case KDocumentTextFileFormat_CSS:{
			mSettings = [[KModeSettings alloc] initWithFileOfName:KModeSettingsFileName_CSS ofType:@"txt"];
			mKeywords = [[KKeywords alloc] initWithFileOfName:KKeywordsFileName_CSS ofType:@"txt" withSeparators:@", \t\r\n"];
			break;
		}
		case KDocumentTextFileFormat_Perl:{
			mSettings = [[KModeSettings alloc] initWithFileOfName:KModeSettingsFileName_Perl ofType:@"txt"];
			mKeywords = [[KKeywords alloc] initWithFileOfName:KKeywordsFileName_Perl ofType:@"txt" withSeparators:@", \t\r\n"];
			break;
		}
		case KDocumentTextFileFormat_PHP:{
			mSettings = [[KModeSettings alloc] initWithFileOfName:KModeSettingsFileName_PHP ofType:@"txt"];
			mKeywords = [[KKeywords alloc] initWithFileOfName:KKeywordsFileName_PHP ofType:@"txt" withSeparators:@", \t\r\n"];
			break;
		}
		case KDocumentTextFileFormat_Ruby:{
			mSettings = [[KModeSettings alloc] initWithFileOfName:KModeSettingsFileName_Ruby ofType:@"txt"];
			mKeywords = [[KKeywords alloc] initWithFileOfName:KKeywordsFileName_Ruby ofType:@"txt" withSeparators:@", \t\r\n"];
			break;
		}
		default: NSLog(@"KColorSyntaxServer::resetSettings - non-supported type."); return;
	}
}

- (void)dealloc
{
	[mSettings release];
	[mKeywords release];
	
	[super dealloc];
}




// language-specific coloring methods.


- (void)validateColorSyntaxPlain:(NSRange)validatedRange;
{	
  if(![mSettings backgroundColor] || ![mSettings baseColor]){
		NSLog(@"KColorSyntaxServer::validateColorSyntaxPlain - invalid color.");
		return;
	}
  
  [mTextStorage addAttribute:NSForegroundColorAttributeName value:[mSettings baseColor] range:validatedRange];
}

- (void)validateColorSyntaxHTML:(NSRange)validatedRange
{
	if(![mSettings backgroundColor] || ![mSettings baseColor]
			|| ![mSettings commentColor] || ![mSettings tagColor]
			|| ![mSettings doublequoteColor] || ![mSettings refColor]){
		NSLog(@"KColorSyntaxServer::validateColorSyntaxHTML - invalid color.");
		return;
	}
	
	if([self checkMultiLineCommentCountWithHead:@"<!--" tail:@"-->"]){
		validatedRange = NSMakeRange(0, [mTextStorage length]);
	}else{
		validatedRange = [[mTextStorage string] lineRangeForRange:validatedRange];
	}
	
	NSDictionary* dict = [KStringUtils rangesOfHMLSyntaxWith:[mTextStorage string]
									range:validatedRange
									keywords:nil];
									
	NSArray* array = [dict objectForKey:@"single_quote"];
	unsigned i;
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings doublequoteColor] range:[[array objectAtIndex:i] range]];
	}
	
	array = [dict objectForKey:@"double_quote"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings doublequoteColor] range:[[array objectAtIndex:i] range]];
	}
	
	array = [dict objectForKey:@"tags"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings tagColor] range:[[array objectAtIndex:i] range]];
	}
	
	array = [dict objectForKey:@"comments"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings commentColor] range:[[array objectAtIndex:i] range]];
	}
	
	array = [dict objectForKey:@"others"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings baseColor] range:[[array objectAtIndex:i] range]];
	}
	
	array = [dict objectForKey:@"refs"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings refColor] range:[[array objectAtIndex:i] range]];
	}
}

- (void)validateColorSyntaxPHP:(NSRange)validatedRange
{
	if(![mSettings backgroundColor] || ![mSettings baseColor]
			|| ![mSettings commentColor] || ![mSettings keywordColor]
			|| ![mSettings doublequoteColor] || ![mSettings variableColor]){
		NSLog(@"KColorSyntaxServer::validateColorSyntaxPHP - invalid color.");
		return;
	}
	unsigned i;
	NSArray* multiCommentRanges = [KStringUtils rangesOfMultiCommentString:[mTextStorage string] head:@"/*" tail:@"*/" range:NSMakeRange(0, [mTextStorage length])];
	for(i = 0; i < [multiCommentRanges count]; i++){
		if([[multiCommentRanges objectAtIndex:i] containsRange:validatedRange]){
			[self setForegroundColorWith:[mSettings commentColor] range:[[multiCommentRanges objectAtIndex:i] range]];
			return;
		}
	}
	
	if([self checkMultiLineCommentCountWithHead:@"/*" tail:@"*/"]){
		validatedRange = NSMakeRange(0, [mTextStorage length]);
	}else{
		validatedRange = [[mTextStorage string] lineRangeForRange:validatedRange];
	}
	//validatedRange = NSMakeRange(0, [mTextStorage length]);
	
	NSDictionary* dict = [KStringUtils rangesOfColorSyntaxWith:[mTextStorage string]
							range:validatedRange
							multiCommentHead:@"/*"
							multiCommentTail:@"*/"
							singleComment:@"//"
							keywords:[mKeywords keywords]
							variableHead:@"$"
							tokenChars:[NSString stringWithFormat:@"%@%@", KTokenCharacters, @"$-"]];
	
	NSArray* array = [dict objectForKey:@"single_quote"];
	
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings doublequoteColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"double_quote"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings doublequoteColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"single_comment"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings commentColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"multi_comment"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings commentColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"others"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings baseColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"variables"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings variableColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"keywords"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings keywordColor] range:[[array objectAtIndex:i] range]];
	}
	
}

- (void)validateColorSyntaxPerl:(NSRange)validatedRange
{
	if(![mSettings backgroundColor] || ![mSettings baseColor]
			|| ![mSettings commentColor] || ![mSettings keywordColor]
			|| ![mSettings doublequoteColor] || ![mSettings variableColor]){
		NSLog(@"KColorSyntaxServer::validateColorSyntaxPerl - invalid color.");
		return;
	}
	
	unsigned i;

	validatedRange = [[mTextStorage string] lineRangeForRange:validatedRange];
	
	NSDictionary* dict = [KStringUtils rangesOfColorSyntaxWith:[mTextStorage string]
							range:validatedRange
							multiCommentHead:nil
							multiCommentTail:nil
							singleComment:@"#"
							keywords:[mKeywords keywords]
							variableHead:@"@$"
							tokenChars:[NSString stringWithFormat:@"%@%@", KTokenCharacters, @"-@$%&:"]];
		
	NSArray* array = [dict objectForKey:@"single_quote"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings doublequoteColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"double_quote"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings doublequoteColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"single_comment"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings commentColor] range:[[array objectAtIndex:i] range]];
	}
	/*array = [dict objectForKey:@"multi_comment"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings commentColor] range:[[array objectAtIndex:i] range]];
	}*/
	array = [dict objectForKey:@"others"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings baseColor] range:[[array objectAtIndex:i] range]];
		//NSLog([[array objectAtIndex:i] string]);
	}
	array = [dict objectForKey:@"variables"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings variableColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"keywords"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings keywordColor] range:[[array objectAtIndex:i] range]];
	}

	
	/*
	validatedRange = [[mTextStorage string] lineRangeForRange:validatedRange];
	
	[mTextStorage addAttribute:NSForegroundColorAttributeName value:[mSettings baseColor] range:validatedRange];
	[mTextStorage addAttribute:NSBackgroundColorAttributeName value:[mSettings backgroundColor] range:validatedRange];
	
	NSArray* wordArray = [mKeywords keywords];
	NSString* tokenCharacters = [NSString stringWithFormat:@"%@%@", KTokenCharacters, @"-@$%&:"];
	NSArray* tokenArray = [KStringUtils rangesOfTokens:[mTextStorage string] range:validatedRange withTokenCharacters:tokenCharacters];
	unsigned i, j;
	NSRange bufRange;
	NSString* bufString;
	for(i = 0; i < [tokenArray count]; i++){
		bufRange = [[tokenArray objectAtIndex:i] range];
		bufString = [[mTextStorage string] substringWithRange:bufRange];
		if([KStringUtils string:bufString startsWith:@"$"]
				|| [KStringUtils string:bufString startsWith:@"@"]
				|| [KStringUtils string:bufString startsWith:@"&"]){ // Variables
			[mTextStorage addAttribute:NSForegroundColorAttributeName value:[mSettings variableColor] range:bufRange];
			
		}else{ // Key words
			for(j = 0; j < [wordArray count]; j++){
				if([[wordArray objectAtIndex:j] isEqualToString:bufString]){
					[mTextStorage addAttribute:NSForegroundColorAttributeName value:[mSettings keywordColor] range:bufRange];
					break;
				}
			}
		}
	}
	
	[self setQuoteAndCommentColorsWithRange:validatedRange comment:@"#"];
	*/
	//[self setMultiLineCommentColorWithHead:@"/*" tail:@"*/"];

}

- (void)validateColorSyntaxRuby:(NSRange)validatedRange
{
	if(![mSettings backgroundColor] || ![mSettings baseColor]
			|| ![mSettings commentColor] || ![mSettings keywordColor]
			|| ![mSettings doublequoteColor] || ![mSettings variableColor]){
		NSLog(@"KColorSyntaxServer::validateColorSyntaxRuby - invalid color.");
		return;
	}
	
	unsigned i;
	NSArray* multiCommentRanges = [KStringUtils rangesOfMultiCommentString:[mTextStorage string] head:@"=begin" tail:@"=end" range:NSMakeRange(0, [mTextStorage length])];
	for(i = 0; i < [multiCommentRanges count]; i++){
		if([[multiCommentRanges objectAtIndex:i] containsRange:validatedRange]){
			[self setForegroundColorWith:[mSettings commentColor] range:[[multiCommentRanges objectAtIndex:i] range]];
			return;
		}
	}
	
	if([self checkMultiLineCommentCountWithHead:@"=begin" tail:@"=end"]){
		validatedRange = NSMakeRange(0, [mTextStorage length]);
	}else{
		validatedRange = [[mTextStorage string] lineRangeForRange:validatedRange];
	}

	//validatedRange = [[mTextStorage string] lineRangeForRange:validatedRange];
	
	NSDictionary* dict = [KStringUtils rangesOfColorSyntaxWith:[mTextStorage string]
							range:validatedRange
							multiCommentHead:@"=begin"
							multiCommentTail:@"=end"
							singleComment:@"#"
							keywords:[mKeywords keywords]
							variableHead:@"$@"
							tokenChars:[NSString stringWithFormat:@"%@%@", KTokenCharacters, @"-@$%&:'?="]];
		
	NSArray* array = [dict objectForKey:@"single_quote"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings doublequoteColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"double_quote"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings doublequoteColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"single_comment"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings commentColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"multi_comment"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings commentColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"others"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings baseColor] range:[[array objectAtIndex:i] range]];
		//NSLog([[array objectAtIndex:i] string]);
	}
	
	array = [dict objectForKey:@"variables"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings variableColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"keywords"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings keywordColor] range:[[array objectAtIndex:i] range]];
	}
}


- (void)validateColorSyntaxCSS:(NSRange)validatedRange
{
	if(![mSettings backgroundColor] || ![mSettings baseColor]
			|| ![mSettings commentColor] || ![mSettings keywordColor]
			|| ![mSettings doublequoteColor] || ![mSettings bracketColor]){
		NSLog(@"KColorSyntaxServer::validateColorSyntaxCSS - invalid color.");
		[mSettings printToStream];
		return;
	}
	
	unsigned i;
	NSArray* multiCommentRanges = [KStringUtils rangesOfMultiCommentString:[mTextStorage string] head:@"/*" tail:@"*/" range:NSMakeRange(0, [mTextStorage length])];
	for(i = 0; i < [multiCommentRanges count]; i++){
		if([[multiCommentRanges objectAtIndex:i] containsRange:validatedRange]){
			[self setForegroundColorWith:[mSettings commentColor] range:[[multiCommentRanges objectAtIndex:i] range]];
			return;
		}
	}
	
	if([self checkMultiLineCommentCountWithHead:@"/*" tail:@"*/"]){
		validatedRange = NSMakeRange(0, [mTextStorage length]);
	}else{
		validatedRange = [[mTextStorage string] lineRangeForRange:validatedRange];
	}
	//validatedRange = NSMakeRange(0, [mTextStorage length]);
	
	NSDictionary* dict = [KStringUtils rangesOfColorSyntaxWith:[mTextStorage string]
							range:validatedRange
							multiCommentHead:@"/*"
							multiCommentTail:@"*/"
							singleComment:@"//"
							keywords:[mKeywords keywords]
							variableHead:nil
							tokenChars:[NSString stringWithFormat:@"%@%@", KTokenCharacters, @":-"]];
	
	NSArray* array = [dict objectForKey:@"single_quote"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings doublequoteColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"double_quote"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings doublequoteColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"single_comment"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings commentColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"multi_comment"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings commentColor] range:[[array objectAtIndex:i] range]];
	}
	array = [dict objectForKey:@"others"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings baseColor] range:[[array objectAtIndex:i] range]];
		//NSLog([[array objectAtIndex:i] string]);
	}
	/*array = [dict objectForKey:@"variables"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings variableColor] range:[[array objectAtIndex:i] range]];
	}*/
	array = [dict objectForKey:@"keywords"];
	for(i = 0; i < [array count]; i++){
		[self setForegroundColorWith:[mSettings keywordColor] range:[[array objectAtIndex:i] range]];
	}
}


- (void)validateColorSyntaxIndexed:(NSRange)validatedRange
{
	if(![mSettings backgroundColor] || ![mSettings baseColor]
			|| ![mSettings indexColor]){
		NSLog(@"KColorSyntaxServer::validateColorSyntaxIndexed - invalid color.");
		return;
	}
	validatedRange = [[mTextStorage string] lineRangeForRange:validatedRange];
	
	[mTextStorage addAttribute:NSForegroundColorAttributeName value:[mSettings baseColor] range:validatedRange];
	[mTextStorage addAttribute:NSBackgroundColorAttributeName value:[mSettings backgroundColor] range:validatedRange];
	
	unsigned i;
	for(i = 0; i < [[mKeywords keywords] count]; i++){
		[self setLineColorWithRange:validatedRange startsWith:[[mKeywords keywords] objectAtIndex:i] color:[mSettings indexColor]];
	}
	
}

- (void)validateColorSyntaxKMarkup:(NSRange)validatedRange
{
	if(![mSettings backgroundColor] || ![mSettings baseColor]
			|| ![mSettings groupColor] || ![mSettings itemColor]){
		NSLog(@"KColorSyntaxServer::validateColorSyntaxKMarkup - invalid color.");
		return;
	}
	validatedRange = [[mTextStorage string] lineRangeForRange:validatedRange];
	
	[mTextStorage addAttribute:NSForegroundColorAttributeName value:[mSettings baseColor] range:validatedRange];
	[mTextStorage addAttribute:NSBackgroundColorAttributeName value:[mSettings backgroundColor] range:validatedRange];
	
	[self setLineColorWithRange:validatedRange startsWith:@"*" color:[mSettings itemColor]];
	[self setLineColorWithRange:validatedRange startsWith:@"@" color:[mSettings groupColor]];

}




// internal use only

- (void)setQuoteAndCommentColorsWithRange:(NSRange)validatedRange comment:(NSString*)comment
{
	NSDictionary* dict = [KStringUtils rangesOfQuotedAndCommentedString:[mTextStorage string] comment:comment range:validatedRange];
	NSArray* singleQuoteArray = [dict objectForKey:@"single_quote"];
	NSArray* doubleQuoteArray = [dict objectForKey:@"double_quote"];
	NSArray* commentArray = [dict objectForKey:@"comment"];
	unsigned i;
	for(i = 0; i < [singleQuoteArray count]; i++){
		[mTextStorage addAttribute:NSForegroundColorAttributeName value:[mSettings doublequoteColor] range:[[singleQuoteArray objectAtIndex:i] range]];
	}
	for(i = 0; i < [doubleQuoteArray count]; i++){
		[mTextStorage addAttribute:NSForegroundColorAttributeName value:[mSettings doublequoteColor] range:[[doubleQuoteArray objectAtIndex:i] range]];
	}
	for(i = 0; i < [commentArray count]; i++){
		[mTextStorage addAttribute:NSForegroundColorAttributeName value:[mSettings commentColor] range:[[commentArray objectAtIndex:i] range]];
	}

}

- (void)setMultiLineCommentColorWithHead:(NSString*)head tail:(NSString*)tail
{
	[self setMultiLineCommentColorWithHead:head tail:tail color:nil ignoreQuotes:NO];
}

- (void)setMultiLineCommentColorWithHead:(NSString*)head tail:(NSString*)tail color:(NSColor*)color ignoreQuotes:(BOOL)ignoreQuotes
{
	NSArray* multiCommentArray = [KStringUtils rangesOfMultiCommentString:[mTextStorage string] head:head tail:tail range:NSMakeRange(0, [[mTextStorage string] length]) ignoreQuotes:ignoreQuotes];
	NSColor* commentColor = color;
	if(!color){ commentColor = [mSettings commentColor]; }
	
	unsigned i;
	unsigned count = [multiCommentArray count];
	for(i = 0; i < count; i++){
		NSRange commentRange = [[multiCommentArray objectAtIndex:i] range];
		[self setForegroundColorWith:commentColor range:commentRange];
	}
}

- (void)setLineColorWithRange:(NSRange)validatedRange startsWith:(NSString*)string color:(NSColor*)color
{
	NSArray* array = [KStringUtils rangesOfLines:[mTextStorage string] range:validatedRange startsWith:string ignoreCase:NO];
	unsigned i;
	
	for(i = 0; i < [array count]; i++){
		[mTextStorage addAttribute:NSForegroundColorAttributeName value:color range:[[array objectAtIndex:i] range]];
	}
}

- (void)setForegroundColorWith:(NSColor*)color range:(NSRange)range
{
	NSRange longestRange;
	NSDictionary* dict = [mTextStorage attributesAtIndex:range.location 
									longestEffectiveRange:&longestRange
									inRange:range];
	if(![KColorUtils isEqualColor:[dict objectForKey:NSForegroundColorAttributeName] andColor:color]
			|| longestRange.length < range.length){
		[mTextStorage addAttribute:NSForegroundColorAttributeName value:color range:range];
		//NSLog(@"edited");
	}
}

- (BOOL)checkMultiLineCommentCountWithHead:(NSString*)head tail:(NSString*)tail
{
	return [self checkMultiLineCommentCountWithHead:head tail:tail ignoreQuotes:NO];
}

- (BOOL)checkMultiLineCommentCountWithHead:(NSString*)head tail:(NSString*)tail ignoreQuotes:(BOOL)ignoreQuotes
{
	NSRange range = NSMakeRange(0, [mTextStorage length]);
	unsigned count = [KStringUtils countMultiCommentString:[mTextStorage string] head:head tail:tail range:range ignoreQuotes:ignoreQuotes];
	
	if(count == mPrevCommentCount){ return NO; }
	
	mPrevCommentCount = count;
	return YES;
}

@end























