#import "KMenuUtils.h"
#import "KStringUtils.h"
#import "KPrefsServer.h"

@implementation KMenuUtils

+ (NSMenuItem*)searchMenuItemOfMenu:(NSMenu*)targetMenu localizedString:(NSString*)localizedString
{
	id menuItem;
	int i;
	for(i = 0; i < [targetMenu numberOfItems]; i++){
		menuItem = [targetMenu itemAtIndex:i];
		if([[menuItem title] isEqualToString:NSLocalizedString(localizedString, @"")]){
			return menuItem;
		}else if([menuItem hasSubmenu]){
			menuItem = [self searchMenuItemOfMenu:[menuItem submenu] localizedString:localizedString];
			if(menuItem){
				return menuItem;
			}
		}
	}
	return nil;
}

+ (void)turnOffAllMenuItemState:(NSMenu*)targetMenu
{
	id menuItem;
	int i;
	for(i = 0; i < [targetMenu numberOfItems]; i++){
		menuItem = [targetMenu itemAtIndex:i];
		[menuItem setState:NSOffState];
		if([menuItem hasSubmenu]){
			[self turnOffAllMenuItemState:[menuItem submenu]];
		}
	}
}

+ (NSMenuItem*)searchMenuItemWithTag:(int)tag targetMenu:(NSMenu*)targetMenu
{
	id menuItem, returnedItem;
	
	int i;
	for(i = 0; i < [targetMenu numberOfItems]; i++){
		menuItem = [targetMenu itemAtIndex:i];
		if([menuItem tag] == tag){ return menuItem; }
		if([menuItem hasSubmenu]){
			returnedItem = [self searchMenuItemWithTag:tag targetMenu:[menuItem submenu]];
			if(returnedItem){ return returnedItem; }
		}
	}
	return nil;
}

+ (NSMenuItem*)getMenuItemWithSubmenu:(NSMenu*)targetMenu
{
	NSMenu* upperMenu = [targetMenu supermenu];
	if(!upperMenu){ return nil; }
	
	id upperMenuItem;
	int i;
	for(i = 0; i < [upperMenu numberOfItems]; i++){
		upperMenuItem = [upperMenu itemAtIndex:i];
		if([upperMenuItem hasSubmenu]){
			if([upperMenuItem submenu] == targetMenu){
				return upperMenuItem;
			}
		}
	}
	return nil;
}


+ (NSMenu*)makeFontMenuHasSelector:(SEL)selector fontArray:(NSArray**)fontArray
{
	/*
	NSMenu* mainMenu = [NSApp mainMenu];
	NSMenu* fontMenu = [[NSMenu alloc] initWithTitle:@"font_menu"];
	NSFontManager* fontManager = [NSFontManager sharedFontManager];
	NSMutableArray* murableFontArray = [[NSMutableArray alloc]initWithCapacity:200];
	
	NSArray* systemFontArray = [fontManager availableFonts];
	NSArray* nonCharacterFontArray = [fontManager availableFontNamesWithTraits:(NSNonstandardCharacterSetFontMask)];
	
	int i, j;
	int tag = 0;
	NSString* fontName;
	for(i = 0; i < [systemFontArray count]; i++){
		fontName = [systemFontArray objectAtIndex:i];
		for(j = 0; j < [nonCharacterFontArray count]; j++){
			if(![fontName isEqualToString:[nonCharacterFontArray objectAtIndex:j]]
				&& ![[fontName substringWithRange:NSMakeRange(0, 1)] isEqualToString:@"."]){
				[mutableFontArray addObject:fontName];
			}
		}
	}
	
	NSArray* resArray = [[NSArray alloc] initWithArray:mutableFontArray];
	fontArray = &resArray;
	[murableFontArray release];
	
	return fontMenu;
	*/
	return nil;
}

+ (NSArray*)attachFontMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSFontManager* fontManager = [NSFontManager sharedFontManager];
	NSMutableArray* mutableFontArray = [[NSMutableArray alloc] initWithCapacity:200];
	
	int i, j;
	int tag = 0;
	NSMenu* memberMenu;
	id fontMenuItem;
	id memberMenuItem;
	NSArray* fontArray, *memberArray;
	NSString* fontName;
	//NSDictionary* dict;
	
	fontArray = [fontManager availableFontFamilies];
	fontArray = [fontArray sortedArrayUsingSelector:@selector(localizedCaseInsensitiveCompare:)];
	for(i = 0; i < [fontArray count]; i++){
		fontName = [fontArray objectAtIndex:i];
		if([[fontName substringWithRange:NSMakeRange(0, 1)] isEqualToString:@"."]){ continue; }
		
		fontMenuItem = [targetMenu addItemWithTitle:[fontManager localizedNameForFamily:fontName face:nil] action:nil keyEquivalent:@""];
		memberMenu = [[NSMenu alloc] initWithTitle:@"member_submenu"];
		memberArray = [fontManager availableMembersOfFontFamily:fontName];
		for(j = 0; j < [memberArray count]; j++){
			memberMenuItem = [[NSMenuItem alloc] initWithTitle:[[memberArray objectAtIndex:j] objectAtIndex:1] action:selector keyEquivalent:@""];
			[memberMenu addItem:[memberMenuItem autorelease]]; // autreleased
			[memberMenuItem setTag:tag];
			[mutableFontArray addObject:[[memberArray objectAtIndex:j] objectAtIndex:0]];
			tag++;
		}
		[fontMenuItem setSubmenu:[memberMenu autorelease]]; // autreleased
	}
	
	NSArray* retArray = [[NSArray alloc] initWithArray:mutableFontArray];
	[mutableFontArray release];
	
	return retArray;
}


+ (NSArray*)attachFontMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector
{
	NSMenu* fontMenu = [[NSMenu alloc] initWithTitle:@"font_submenu"];
	[targetMenuItem setSubmenu:[fontMenu autorelease]]; // autoreleased
	
	return [self attachFontMenuToMenu:fontMenu selector:selector];
}

+ (NSArray*)attachFlattedFontMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSFontManager* fontManager = [NSFontManager sharedFontManager];
	NSMutableArray* mutableFontArray = [[NSMutableArray alloc] initWithCapacity:200];
	
	int i;
	int tag = 0;
	id fontMenuItem;
	NSArray* fontArray;
	NSString* fontName;
	
	fontArray = [fontManager availableFonts];
	fontArray = [fontArray sortedArrayUsingSelector:@selector(localizedCaseInsensitiveCompare:)];
	for(i = 0; i < [fontArray count]; i++){
		fontName = [fontArray objectAtIndex:i];
		if([[fontName substringWithRange:NSMakeRange(0, 1)] isEqualToString:@"."]){ continue; }
		
		fontMenuItem = [targetMenu addItemWithTitle:[fontManager localizedNameForFamily:fontName face:nil] action:selector keyEquivalent:@""];
		[fontMenuItem setTag:tag];
		[mutableFontArray addObject:fontName];
		tag++;
	}
	
	NSArray* retArray = [[NSArray alloc] initWithArray:mutableFontArray];
	[mutableFontArray release];
	
	return retArray;

}

+ (NSArray*)attachFlattedFontMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector
{
	NSMenu* fontMenu = [[NSMenu alloc] initWithTitle:@"font_submenu"];
	[targetMenuItem setSubmenu:[fontMenu autorelease]]; // autoreleased
	
	return [self attachFlattedFontMenuToMenu:fontMenu selector:selector];
}



+ (void)attachTagMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSFileManager* manager = [NSFileManager defaultManager];
	NSString* dirName = [NSHomeDirectory() stringByAppendingPathComponent:KPreferenceDirectory];
	NSString* tagFileName = [dirName stringByAppendingPathComponent:KTagFileName];
	
	if(![manager fileExistsAtPath:tagFileName]){
		NSBundle* bundle = [NSBundle mainBundle];
		tagFileName = [bundle pathForResource:@"html_tags" ofType:@"txt"];
		if(!tagFileName){ NSLog(@"KMenuUtils : tagFileName == nil"); return; }
	}
	
	NSString* tagFileContents = [NSString stringWithContentsOfFile:tagFileName];
	if(!tagFileContents){ NSLog(@"KMenuUtils : tagFileContents == nil"); return; }
	
	NSArray* commandArray = [KStringUtils string:tagFileContents componentsSeparatedByCharacters:@"\r\n"];
	//NSArray* commandArray = [tagFileContents componentsSeparatedByString:@"\r"];
	//NSLog([NSString stringWithFormat:@"command count = %d", [commandArray count]]);
	unsigned i;
	NSString* commandLine;
	NSMenu* menu = targetMenu;
	//KMenuItem* menuItem;
        id menuItem;
	
	for(i = 0; i < [commandArray count]; i++){
		commandLine = [KStringUtils removeSpacesFromHeadOf:[commandArray objectAtIndex:i]];
		if([KStringUtils string:commandLine startsWith:@"//"]) { continue; }
		if([commandLine isEqualToString:@""]){ continue; }
		
		NSDictionary* dict = [KStringUtils resolveCommand:commandLine];
		//NSLog([NSString stringWithFormat:@"command = %@, string = %@", [dict objectForKey:@"command"], [dict objectForKey:@"string"]]);	
		if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"SHIFT"]){
			menuItem = [menu addItemWithTitle:[dict objectForKey:@"string"] action:nil keyEquivalent:@""];
			menu = [[NSMenu alloc] initWithTitle:@""];
			[menuItem setSubmenu:[menu autorelease]]; // autoreleased
			
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"UNSHIFT"]){
			NSMenu* bufMenu = [menu supermenu];
			if(bufMenu){ menu = bufMenu; }
			
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"NAME"]){
			menuItem = [[NSMenuItem alloc] initWithTitle:[dict objectForKey:@"string"] action:selector keyEquivalent:@""];
			[menuItem setRepresentedObject:[NSMutableDictionary dictionaryWithCapacity:10]];
			[menu addItem:[menuItem autorelease]]; // autoreleased
			
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"PRE"]){
			[[menuItem representedObject] setObject:[dict objectForKey:@"string"] forKey:@"pre"];
		
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"POST"]){
			[[menuItem representedObject] setObject:[dict objectForKey:@"string"] forKey:@"post"];
		
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"SINGLE"]){
			[[menuItem representedObject] setObject:[dict objectForKey:@"string"] forKey:@"single"];
		
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"SELECT"]){
			[[menuItem representedObject] setObject:[[dict objectForKey:@"string"] lowercaseString] forKey:@"select"];
		
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"MOD"]){
			NSString* mod = [dict objectForKey:@"string"];
			unsigned mask = NSCommandKeyMask;
			if([KStringUtils string:[mod uppercaseString] contains:@"OPT"]){ mask |= NSAlternateKeyMask; }
			if([KStringUtils string:[mod uppercaseString] contains:@"SHIFT"]){ mask |= NSShiftKeyMask; }
			if([KStringUtils string:[mod uppercaseString] contains:@"CTRL"]){ mask |= NSControlKeyMask; }
			[menuItem setKeyEquivalentModifierMask:mask];
		
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"KEY"]){
			NSString* key = [dict objectForKey:@"string"];
			if([[key uppercaseString] isEqualToString:@"RETURN"]){
				key = [NSString stringWithString:@"\n"];
			}else if([[key uppercaseString] isEqualToString:@"TAB"]){
				key = [NSString stringWithString:@"\t"];
			}else if([[key uppercaseString] isEqualToString:@"ESC"]){
				//key = [NSString stringWithString:@"\r"];
				//currently do nothing...
			}
			[menuItem setKeyEquivalent:[key lowercaseString]];
		}
	}
	
}

+ (void)attachTagMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector
{
	NSMenu* tagMenu = [[NSMenu alloc] initWithTitle:@"tag_submenu"];
	[targetMenuItem setSubmenu:[tagMenu autorelease]];
	[KMenuUtils attachTagMenuToMenu:tagMenu selector:selector];
}


+ (void)attachScriptsMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSFileManager* manager = [NSFileManager defaultManager];
	NSString* addonsDirName = [[NSHomeDirectory() stringByAppendingPathComponent:KPreferenceDirectory]
									stringByAppendingPathComponent:KAddOnsDirectoryName];
	
	if(![manager fileExistsAtPath:addonsDirName]){
		[targetMenu addItemWithTitle:@"no item" action:nil keyEquivalent:@""];
		return;
	}
	[self constructScriptMenu:targetMenu dir:addonsDirName selector:(SEL)selector];
	
}

+ (void)attachScriptsMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector
{
	NSMenu* menu = [[NSMenu alloc] initWithTitle:@"no_title"];
	[self attachScriptsMenuToMenu:menu selector:selector];
	[targetMenuItem setSubmenu:[menu autorelease]]; // autoreleased
}





// internal use only.
+ (void)constructScriptMenu:(NSMenu*)targetMenu dir:(NSString*)dirPath selector:(SEL)selector
{
	NSFileManager* manager = [NSFileManager defaultManager];
	//NSDirectoryEnumerator* enumerator = [manager enumeratorAtPath:dirPath];
	NSArray* contents = [manager directoryContentsAtPath:dirPath];
	NSString* filePath;
	NSString* fileName;
	NSString* key;
	NSMenuItem* menuItem;
	NSMenu* menu;
	BOOL isDir;
	unsigned i;
	NSArray* components;
	
	for(i = 0; i < [contents count]; i++){
		fileName = [contents objectAtIndex:i];
		if([KStringUtils string:fileName startsWith:@"."]){ continue; }
		
		filePath = [dirPath stringByAppendingPathComponent:fileName];
		//NSLog(filePath);
		if([manager fileExistsAtPath:filePath isDirectory:&isDir]){
			fileName = [manager displayNameAtPath:filePath];
			components = [fileName componentsSeparatedByString:@"."];
			components = [[components objectAtIndex:0] componentsSeparatedByString:@"-"];
			
			menuItem = [[NSMenuItem alloc] initWithTitle:fileName action:selector keyEquivalent:@""];
			[targetMenu addItem:[menuItem autorelease]]; //autoreleased
			if(isDir){
				menu = [[NSMenu alloc] initWithTitle:@""];
				[menuItem setSubmenu:[menu autorelease]]; // autoreleased
				[self constructScriptMenu:menu dir:filePath selector:selector];
			}else{
				[menuItem setRepresentedObject:filePath];
				if([components count] >= 2){
					key = [components objectAtIndex:1];
					[menuItem setKeyEquivalent:key];
					[menuItem setKeyEquivalentModifierMask:(NSCommandKeyMask | NSAlternateKeyMask)];
				}
			}
		}
	}
	if(![targetMenu numberOfItems]){
		[targetMenu addItemWithTitle:@"no item" action:nil keyEquivalent:@""];
	}
}

+ (void)constructTagMenu:(NSMenu*)targetMenu selector:(SEL)selector tagArray:(NSArray*)tagArray
{
	NSMenuItem* menuItem;
	NSMenu* subMenu;
	NSDictionary* dict;
	NSArray* subArray;
	NSString* title, *key, *modifiers;
	unsigned int modifiersMask = NSCommandKeyMask;
	unsigned i;
	for(i = 0; i < [tagArray count]; i++){
		dict = [tagArray objectAtIndex:i];
		title = [dict objectForKey:@"title"];
		key = [dict objectForKey:@"key"];
		modifiers = [dict objectForKey:@"modifiers"];
		
		if(!key){ key = [NSString stringWithString:@""]; }
		if([key isEqualToString:@"RETURN"]){
			key = [NSString stringWithString:@"\r"];
		}
		if([KStringUtils string:modifiers contains:@"option"]){
			modifiersMask |= NSAlternateKeyMask;
		}
		//modifiersMask &= ~NSShiftKeyMask;
		
		subArray = [dict objectForKey:@"sub_array"];
		menuItem = [[NSMenuItem alloc] initWithTitle:title action:selector keyEquivalent:key];
		[menuItem setKeyEquivalentModifierMask:modifiersMask];
		if(subArray){
			subMenu = [[NSMenu alloc] initWithTitle:@""];
			[self constructTagMenu:subMenu selector:selector tagArray:subArray];
			[menuItem setSubmenu:[subMenu autorelease]]; //autoreleased 
		}else{
			[menuItem setRepresentedObject:dict];
		}
		[targetMenu addItem:[menuItem autorelease]]; // autoreleased
	}
	//NSLog([NSString stringWithFormat:@"constructTagMenu:count = %d", [tagArray count]]);
}


@end










