//
//  KRangeObj.m
//  KEdit
//
//  Created by Karino, Masatsugu on Tue May 06 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "KRangeObj.h"


@implementation KRangeObj

+ (id)rangeObj
{
	return [[[self alloc] init] autorelease];
}

+ (id)rangeObjWithRange:(NSRange)range
{
	return [[[self alloc] initWithRange:range] autorelease];
}

+ (id)rangeObjWithLocation:(unsigned)location andLength:(unsigned)length
{
	return [[[self alloc] initWithLocation:location andLength:length] autorelease];
}

+ (id)rangeObjWithRangeObj:(KRangeObj*)range
{
	return [[[self alloc] initWithRangeObj:range] autorelease];
}

+ (id)rangeObjWithString:(NSString*)string
{
	return [[[self alloc] initWithString:string] autorelease];
}

- (id)init
{
	mLocation = mLength = 0;
	return self;
}

- (id)initWithRange:(NSRange)range
{
	mLocation = range.location;
	mLength = range.length;
	return self;
}

- (id)initWithLocation:(unsigned)location andLength:(unsigned)length
{
	mLocation = location;
	mLength = length;
	return self;
}

- (id)initWithRangeObj:(KRangeObj*)range
{
	mLocation = range->mLocation;
	mLength = range->mLength;
	return self;
}

- (id)initWithString:(NSString*)string
{
	NSRange range = NSRangeFromString(string);
	mLocation = range.location;
	mLength = range.length;
	return self;
}


- (BOOL)isEqualToRange:(NSRange)range
{
	return ((mLocation == range.location) && (mLength == range.length));
}

- (BOOL)isEqualTo:(KRangeObj*)range
{
	return ((mLocation == range->mLocation) && (mLength == range->mLength));
}

- (BOOL)containsLocation:(unsigned)location
{
	return (mLocation <= location) && (location <= mLocation + mLength);
}

- (BOOL)containsRange:(NSRange)range
{
	return (mLocation <= range.location) && (mLocation + mLength >= range.location + range.length);
}

- (BOOL)containsRangeObj:(KRangeObj*)range
{
	return [self containsRange:[range range]];
}

- (BOOL)isContainedWithRangeObjs:(NSArray*)array
{
	unsigned i;
	BOOL isContained = NO;
	for(i = 0; i < [array count]; i++){
		if([[array objectAtIndex:i] containsRangeObj:self]){
			isContained = YES;
			break;
		}
	}
	return isContained;
}


- (id)initWithCoder:(NSCoder*)decoder
{
	[decoder decodeValueOfObjCType:"i" at:&mLocation];
	[decoder decodeValueOfObjCType:"i" at:&mLength];
	return self;
}

- (void)encodeWithCoder:(NSCoder*)encoder
{
	[encoder encodeValueOfObjCType:"i" at:&mLocation];
	[encoder encodeValueOfObjCType:"i" at:&mLength];
}


- (id)unionRange:(NSRange)range
{
	NSRange retRange = NSUnionRange([self range], range);
	return [KRangeObj rangeObjWithRange:retRange];
}

- (id)unionRangeObj:(KRangeObj*)range
{
	NSRange retRange = NSUnionRange([self range], [range range]);
	return [KRangeObj rangeObjWithRange:retRange];
}

- (id)intersectionRange:(NSRange)range
{
	NSRange retRange = NSIntersectionRange([self range], range);
	return [KRangeObj rangeObjWithRange:retRange];
}

- (id)intersectionRangeObj:(KRangeObj*)range
{
	NSRange retRange = NSIntersectionRange([self range], [range range]);
	return [KRangeObj rangeObjWithRange:retRange];
}

/*
- (NSArray*)complementRangeObjs:(NSArray*)rangeArray
{
	NSArray* array = [rangeArray sortedArrayUsingSelector:@selector(locationCompare:)];
	unsigned count = [array count];
	NSMutableArray* muArray = [NSMutableArray arrayWithCapacity:(count + 1)];
	
	unsigned i;
	NSRange prevRange = NSMakeRange(0, 0);
	NSRange currentRange, bufRange;
	
	for(i = 0; i < count; i++){
		currentRange = [[array objectAtIndex:i] range];
		bufRange.location = NSMaxRange(prevRange);
		bufRange.length = currentRange.location - bufRange.location;
		[muArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
		prevRange = currentRange;
	}
	
	
	return muArray;
}*/

- (unsigned)maxRange
{
	return mLocation + mLength;
}

- (id)shiftRangeObj:(unsigned)shiftLoc
{
	if(shiftLoc > mLength){
		return nil;
	}
	mLocation += shiftLoc;
	mLength -= shiftLoc;
	
	return self;
}

- (NSComparisonResult)locationCompare:(KRangeObj*)range
{
	if(mLocation == range->mLocation){
		return NSOrderedSame;
	}else if(mLocation < range->mLocation){
		return NSOrderedAscending;
	}
	
	return NSOrderedDescending;
}


- (void)setLocation:(unsigned)location { mLocation = location; }
- (void)setLength:(unsigned)length { mLength = length; }
- (void)setRange:(NSRange)range { mLocation = range.location; mLength = range.length; }
- (unsigned)location { return mLocation; }
- (unsigned)length { return mLength; }
- (NSRange)range { return NSMakeRange(mLocation, mLength); }
- (NSString*)string { return NSStringFromRange(NSMakeRange(mLocation, mLength)); }

@end










