//
//  KSettingFile.h
//  KEdit
//
//  Created by Karino, Masatsugu on Thu May 22 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

// default setting items
// COLORs
#define KModeSettingsDefault_BaseStringColor @"base_string_color"
#define KModeSettingsDefault_BackgroundColor @"background_color"
#define KModeSettingsDefault_IsWordWrap @"is_word_wrap"
#define KModeSettingsDefault_KeywordColor @"keyword_color"
#define KModeSettingsDefault_CommentColor @"comment_color"
#define KModeSettingsDefault_SingleQuoteColor @"singlequote_color"
#define KModeSettingsDefault_DoubleQuoteColor @"doublequote_color"
#define KModeSettingsDefault_VariableColor @"variable_color"
#define KModeSettingsDefault_ItemColor @"item_color"
#define KModeSettingsDefault_GroupColor @"group_color"
#define KModeSettingsDefault_BracketColor @"bracket_color"
#define KModeSettingsDefault_IndexColor @"indexed_line_color"
#define KModeSettingsDefault_TagColor @"tag_color"
#define KModeSettingsDefault_RefColor @"ref_color"
// BOOLs
#define KModeSettingsDefault_IsWordWrapped @"is_word_wrapped"
// FLOATs
#define KModeSettingsDefault_FontSize @"font_size"


@class NSColor;

// Base Class of Setting Files.
@interface KSettingFile : NSObject {
	NSString* mBundlePath; // a full path of the setting file in Bundle.
	NSString* mUserPath; // a full path of the setting file in the 'setting' directory, or nil.
	
	NSString* mFileName; // a partial file name passed as fileName.
}
- (id)initWithFileOfName:(NSString*)fileName ofType:(NSString*)type;
+ (id)settingsWithFileOfName:(NSString*)fileName ofType:(NSString*)type;
- (BOOL)analyze; // virtual ... (re)construct class-specific data object.
- (NSString*)fileName; // return mFileName.
@end


// Mode Settings.
// serve setting strings as a dictionary.
@interface KModeSettings : KSettingFile {
	NSDictionary* mSettings;
}
- (NSDictionary*)settings;
- (id)settingForKey:(NSString*)key;
- (NSColor*)colorForKey:(NSString*)key;

// for debugging
- (void)printToStream;

// colors for general purpose
- (NSColor*)backgroundColor;
- (NSColor*)baseColor;
- (NSColor*)commentColor;
- (NSColor*)keywordColor;
- (NSColor*)doublequoteColor;
- (NSColor*)singlequoteColor;
- (NSColor*)variableColor;

// colors for specific purpose
- (NSColor*)groupColor;
- (NSColor*)itemColor;
- (NSColor*)bracketColor;
- (NSColor*)indexColor;
- (NSColor*)tagColor;
- (NSColor*)refColor;

// floats
- (float)fontSize;

// bools
- (BOOL)isWordWrapped;

// internal use.
- (BOOL)analyze;
@end


// Keywords.
// serve word array separated with passed characters.
@interface KKeywords : KSettingFile {
	NSArray* mKeywords;
	NSString* mSeparators;
}
- (id)initWithFileOfName:(NSString*)fileName ofType:(NSString*)type withSeparators:(NSString*)characters;
+ (id)keywordsWithFileOfName:(NSString*)fileName ofType:(NSString*)type withSeparators:(NSString*)characters;
// don't use "*WithFileOfName:ofType:" functions directly.
- (NSArray*)keywords;

// internal use.
- (BOOL)analyze;
@end


















