#import "KStringUtils.h"
#import "KRangeObj.h"

// C lang.
int unistrncmp(unichar* target, unichar* sub, unsigned length)
{
	unsigned i;
	for(i = 0; i < length; i++){
		if(target[i] < sub[i]){ return 1; }
		if(target[i] > sub[i]){ return -1; }
	}
	return 0;
}

bool uni_is_return_char(unichar target)
{
	if(target == NSCarriageReturnCharacter
			|| target == NSLineSeparatorCharacter
			|| target == NSParagraphSeparatorCharacter
			|| target == NSNewlineCharacter){
		return true;
	}
	return false;
}

// for kunistr struct.
kunistr uni_construct_with_string(NSString* string)
{
	kunistr str;
	str.length = str.mem_size = [string length];
	str.string = (unichar*)malloc(str.mem_size * sizeof(unichar));
	[string getCharacters:(unichar*)str.string];
	return str;
}

kunistr uni_construct_with_size(unsigned size)
{
	kunistr str;
	str.mem_size = size;
	str.string = (unichar*)malloc(size * sizeof(unichar));
	str.length = 0;
	return str;
}

void uni_destruct(kunistr target)
{
	free(target.string);
	target.mem_size = target.length = 0;
}

void uni_print_to_stream(kunistr target)
{
	NSLog([NSString stringWithCharacters:target.string length:target.length]);
}

bool uni_is_equal(kunistr str1, kunistr str2)
{
	if(str1.length != str2.length){ return false; }
	unsigned i;
	for(i = 0; i < str1.length; i++){
		if(str1.string[i] != str2.string[i]){ return false; }
	}
	return true;
}

bool uni_does_contain_char(kunistr str, unichar sub)
{
	unsigned i;
	for(i = 0; i < str.length; i++){
		if(str.string[i] == sub){ return true; }
	}
	return false;
}

int uni_offset_of_char(kunistr str, unichar sub)
{
	int i;
	for(i = 0; i < str.length; i++){
		if(str.string[i] == sub){
			return i;
		}
	}
	return -1;
}

void uni_append_str(kunistr* str1, kunistr* str2){
	unsigned newLength = str1->length + str2->length;
	unsigned i;
	
	str1->string = (unichar*)realloc(str1->string, newLength);
	
	for(i = 0; i < str2->length; i++){
		str1->string[str1->length + i] = str2->string[i];
	}
	str1->length = newLength;
}

void uni_append_char(kunistr* str, unichar c)
{
	unsigned newLength = str->length + 1;	
	str->string = (unichar*)realloc(str->string, newLength);
	str->string[str->length] = c;
	str->length = newLength;
}

kunistr* uni_construct_with_array(NSArray* array)
{
	if(!array){ return nil; }
	
	unsigned count = [array count];
	kunistr* list = (kunistr*)malloc(sizeof(kunistr) * (count + 1));
	unsigned i;
	for(i = 0; i < count; i++){
		list[i] = uni_construct_with_string([array objectAtIndex:i]);
	}
	list[count] = uni_construct_with_size(0);
	
	return list;
}

void uni_destruct_array(kunistr* array)
{
	if(!array){ return; }
	
	unsigned i = 0;
	while(array[i].length){
		uni_destruct(array[i]);
		i++;
	}
	free(array);
}

void uni_print_to_stream_array(kunistr* array)
{
	unsigned i = 0;
	while(array[i].length){
		uni_print_to_stream(array[i]);
		i++;
	}
}



// Obj-C lang.
@implementation KStringUtils

+ (unsigned)countLinesOfString:(NSString*)string
{
	unsigned stringLength = [string length];
	if(stringLength == 0){ return 0; }
	
	return [KStringUtils countLinesOfString:string range:NSMakeRange(0, stringLength)];
}

+ (unsigned)countLinesOfString:(NSString*)string range:(NSRange)range
{
	kunistr substr = uni_construct_with_string([string substringWithRange:range]);
	unsigned lineCount = 1;
	unsigned i;
	for(i = 0; i < substr.length; i++){
		if(uni_is_return_char(substr.string[i])){ lineCount++; }
	}
	uni_destruct(substr);
	
	return lineCount;
}

+ (NSDictionary*)countReturns:(NSString*)string
{
	// 'total' full count of cr/lf/ls/ps
	// 'cr', 'lf', 'ls', 'ps'
	// 'crlf' is count of cr&lf sequence.
	
	unsigned aCR, aLF, aCRLF, aLS, aPS;
	aCR = aLF = aCRLF = aLS = aPS = 0;
	
	unsigned length = [string length];
	unichar* uniChar = (unichar*)malloc(length * sizeof(unichar));
	[string getCharacters:uniChar];
	unsigned i;
	unichar prevChar = 0;
	for(i = 0; i < length; i++){
		switch(uniChar[i]){
			case NSCarriageReturnCharacter: aCR++; break;
			case NSLineSeparatorCharacter: aLS++; break;
			case NSParagraphSeparatorCharacter: aPS++; break;
			case NSNewlineCharacter: {
					aLF++;
					if(prevChar == NSCarriageReturnCharacter){ aCRLF++; }
					break;
				}
			default: ; // do nothing.
		}
		prevChar = uniChar[i];
	}
	free(uniChar);
	
	NSLog([NSString stringWithFormat:@"cr = %d, lf = %d, crlf = %d, ls = %d, ps = %d", aCR, aLF, aCRLF, aLS, aPS]);
	
	NSDictionary* dict = [[NSDictionary alloc] initWithObjectsAndKeys:
							[NSNumber numberWithInt:aCR],   @"cr",
							[NSNumber numberWithInt:aLF],   @"lf",
							[NSNumber numberWithInt:aCRLF], @"crlf",
							[NSNumber numberWithInt:aLS],   @"ls",
							[NSNumber numberWithInt:aPS],   @"ps",
							nil];
	return [dict autorelease];
}

+ (NSString*)convertReturnsToLF:(NSString*)string from:(KReturnCodeType*)type
{
	NSString* retString;
	
	NSArray* array = [string componentsSeparatedByString:@"\r\n"];
	if([array count] >= 2){
		if(type){ *type = KCRLF_type; }
		retString = [array componentsJoinedByString:@"\n"];
		return [self replaceString:retString search:@"\r" replace:@"\n"];
	}
	array = [string componentsSeparatedByString:@"\r"];
	if([array count] >= 2){
		if(type){ *type = KCR_type; }
		return [array componentsJoinedByString:@"\n"];
	}
	
	if(type){ *type = KLF_type; }
	return [NSString stringWithString:string];
}

+ (NSString*)convertReturnsFromLF:(NSString*)string to:(KReturnCodeType)type
{
	NSString* separator;
	if(type == KLF_type){ return [NSString stringWithString:string]; }
	
	if(type == KCR_type){ separator = @"\r"; }
	else if(type == KCRLF_type){ separator = @"\r\n"; }
	return [[string componentsSeparatedByString:@"\n"] componentsJoinedByString:separator];
}

+ (NSString*)relativePathFrom:(NSString*)source to:(NSString*)destination
{	
	if([source rangeOfString:@"/"].location != 0){ return @"source is not a file name."; }
	if([destination rangeOfString:@"/"].location != 0){ return @"destination is not a file name"; }
	
	NSRange offsetRange;
	
	NSString* currentString = [NSString stringWithString:source];
	NSMutableString* newString = [[NSMutableString alloc] initWithString:@""];
	
	while(YES){
		currentString = [[NSString alloc] initWithString:[currentString stringByDeletingLastPathComponent]];
		offsetRange = [destination rangeOfString:currentString];
		if(offsetRange.location == 0){
			if([newString isEqualToString:@""]){ [newString appendString:@"./"]; }
			[newString appendString:[destination substringWithRange:NSMakeRange(offsetRange.length + 1, [destination length] - offsetRange.length - 1)]];
			break;
		}
		[newString appendString:@"../"];
	}
	
	return [newString autorelease];
}

+ (BOOL)string:(NSString*)targetString startsWith:(NSString*)string
{
	if([targetString rangeOfString:string].location == 0){ return YES; }
	return NO;
}

+ (BOOL)string:(NSString*)targetString endsWith:(NSString*)string
{
	if([targetString rangeOfString:string].location == ([targetString length] - [string length])){
		return YES;
	}
	return NO;
}

+ (BOOL)string:(NSString*)targetString contains:(NSString*)string
{
	if([targetString rangeOfString:string].location == NSNotFound){ return NO; }
	return YES;
}


+ (NSString*)removeSpacesFromHeadOf:(NSString*)string
{
	NSString* buffer;
	unsigned i;
	for(i = 0; i < [string length]; i++){
		buffer = [string substringWithRange:NSMakeRange(i, 1)];
		if(![buffer isEqualToString:@"\t"] && ![buffer isEqualToString:@" "]){
			break;
		}
	}
	return [string substringWithRange:NSMakeRange(i, [string length] - i)];
}

+ (NSArray*)string:(NSString*)targetString componentsSeparatedByCharacters:(NSString*)characters
{
	NSCharacterSet* charSet = [NSCharacterSet characterSetWithCharactersInString:characters];
	NSScanner* scanner = [NSScanner scannerWithString:targetString];
	NSString* buffer;
	NSMutableArray* array = [[NSMutableArray alloc] initWithCapacity:100];
	
	while(![scanner isAtEnd]){
		if([scanner scanUpToCharactersFromSet:charSet intoString:&buffer]){
			[array addObject:[NSString stringWithString:buffer]];
		}
		[scanner scanCharactersFromSet:charSet intoString:nil];
	}
	
	return [array autorelease];
}

+ (NSArray*)linesWithoutReturnsOfString:(NSString*)target range:(NSRange)range
{
	NSArray* rangeArray = [KStringUtils rangesOfLinesWithoutReturns:target range:range];
	NSMutableArray* muArray = [NSMutableArray arrayWithCapacity:[rangeArray count]];
	unsigned i;
	for(i = 0; i < [rangeArray count]; i++){
		[muArray addObject:[target substringWithRange:[[rangeArray objectAtIndex:i] range]]];
	}
	return muArray;
}

+ (NSArray*)string:(NSString*)targetString bindedWith:(NSString*)string
{
	NSArray* array = [targetString componentsSeparatedByString:string];
	NSMutableArray* muArray = [[NSMutableArray alloc] initWithCapacity:10];
	unsigned i;
	unsigned count = [array count]/2;
	for(i = 0; i < count; i++){
		[muArray addObject:[array objectAtIndex:(2 * i + 1)]];
	}
	return [muArray autorelease];
}

+ (NSArray*)string:(NSString*)targetString bindedWith:(NSString*)from andWith:(NSString*)to
{
	NSMutableArray* array = [[NSMutableArray alloc] initWithCapacity:100];
	NSArray* bufArray1, *bufArray2;
	
	bufArray1 = [targetString componentsSeparatedByString:from];
	int i;
	if([bufArray1 count] >= 2){
		for(i = 1; i < [bufArray1 count]; i++){
			bufArray2 = [[bufArray1 objectAtIndex:i] componentsSeparatedByString:to];
			if([bufArray2 count] > 0){
				[array addObject:[bufArray2 objectAtIndex:0]];
			}
		}
	}
	
	return [array autorelease];
}

+ (NSArray*)rangesOfTokens:(NSString*)targetString
{
	return [KStringUtils rangesOfTokens:targetString withTokenCharacters:KTokenCharacters];
}

+ (NSArray*)rangesOfTokens:(NSString*)targetString withTokenCharacters:(NSString*)characters
{
	return [KStringUtils rangesOfTokens:targetString range:NSMakeRange(0, [targetString length]) withTokenCharacters:characters];

}

+ (NSArray*)rangesOfTokens:(NSString*)target range:(NSRange)range withTokenCharacters:(NSString*)characters
{
	NSMutableArray* muArray = [NSMutableArray arrayWithCapacity:100];
	kunistr substr = uni_construct_with_string([target substringWithRange:range]);
	kunistr tokenstr = uni_construct_with_string(characters);
	
	BOOL isInToken = NO;
	BOOL isTokenChar;
	NSRange bufRange;
	unsigned i, j;
	
	for(i = 0; i < range.length; i++){
		isTokenChar = NO;
		for(j = 0; j < tokenstr.length; j++){
			if(substr.string[i] == tokenstr.string[j]){
				isTokenChar = YES;
				break;
			}
		}
		if(isTokenChar && !isInToken){
			isInToken = YES;
			bufRange.location = i + range.location;
		}else if(!isTokenChar && isInToken){
			isInToken = NO;
			bufRange.length = i - bufRange.location + range.location;
			[muArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
		}
	}
	if(isInToken){
		bufRange.length = NSMaxRange(range) - bufRange.location;
		[muArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
	}
	
	uni_destruct(tokenstr);
	uni_destruct(substr);
	
	return muArray;
}

+ (NSArray*)rangesOfTokens:(NSString*)target range:(NSRange)range startsWith:(NSString*)string
{
	return [KStringUtils rangesOfTokens:target range:range startsWith:string withTokenCharacters:KTokenCharacters];
}

+ (NSArray*)rangesOfTokens:(NSString*)target range:(NSRange)range startsWith:(NSString*)string withTokenCharacters:(NSString*)characters
{
	unsigned i;
	BOOL isInToken = NO;
	unsigned offset = range.location;
	NSMutableArray* array = [[NSMutableArray alloc] initWithCapacity:100];
	NSRange bufRange;
	
	for(i = range.location; i < NSMaxRange(range); i++){
		if([KStringUtils string:characters contains:[target substringWithRange:NSMakeRange(i, 1)]]){
			if(!isInToken){
				isInToken = YES;
				offset = i;
			}
		}else{
			if(isInToken){
				isInToken = NO;
				bufRange = NSMakeRange(offset, i - offset);
				if([KStringUtils string:[target substringWithRange:bufRange] startsWith:string]){
					[array addObject:[KRangeObj rangeObjWithRange:bufRange]];
				}
			}
		}
	}
	if(isInToken){
		bufRange = NSMakeRange(offset, NSMaxRange(range) - offset);
		if([KStringUtils string:[target substringWithRange:bufRange] startsWith:string]){
			[array addObject:[KRangeObj rangeObjWithRange:bufRange]];
		}
	}
	return [array autorelease];
}

+ (NSArray*)rangesOfStrings:(NSString*)target range:(NSRange)range startsWith:(NSString*)from endsWith:(NSString*)to
{
	unsigned endOffset = range.location + range.length;
	unsigned mask = NSCaseInsensitiveSearch;
	NSRange currentRange = range;
	if(currentRange.location == NSNotFound){ currentRange = NSMakeRange(0, [target length]); }
	
	NSRange fromRange, toRange, retRange;
	NSMutableArray* muArray = [[NSMutableArray alloc] initWithCapacity:10];
	
	while(YES){
		fromRange = [target rangeOfString:from options:mask range:currentRange];
		if(fromRange.location == NSNotFound){ break; }
		retRange.location = fromRange.location;
		
		currentRange.location = retRange.location + fromRange.length;
		currentRange.length = endOffset - currentRange.location;
		
		toRange = [target rangeOfString:to options:mask range:currentRange];
		if(toRange.location == NSNotFound){ break; }
		retRange.length = toRange.location - retRange.location + toRange.length;
		
		[muArray addObject:[KRangeObj rangeObjWithRange:retRange]];
		
		currentRange.location = toRange.location + toRange.length;
		currentRange.length = endOffset - currentRange.location;
	}
	
	return [muArray autorelease];
}

+ (NSArray*)rangesOfStrings:(NSString*)target range:(NSRange)range bindedWith:(NSString*)from andWith:(NSString*)to
{
	unsigned endOffset = range.location + range.length;
	unsigned mask = NSCaseInsensitiveSearch;
	NSRange currentRange = range;
	if(currentRange.location == NSNotFound){ currentRange = NSMakeRange(0, [target length]); }
	
	NSRange fromRange, toRange, retRange;
	NSMutableArray* muArray = [[NSMutableArray alloc] initWithCapacity:10];
	
	while(YES){
		fromRange = [target rangeOfString:from options:mask range:currentRange];
		if(fromRange.location == NSNotFound){ break; }
		retRange.location = fromRange.location + fromRange.length;
		
		currentRange.location = retRange.location;
		currentRange.length = endOffset - retRange.location;
		
		toRange = [target rangeOfString:to options:mask range:currentRange];
		if(toRange.location == NSNotFound){ break; }
		retRange.length = toRange.location - retRange.location;
		
		[muArray addObject:[KRangeObj rangeObjWithRange:retRange]];
		
		currentRange.location = toRange.location + toRange.length;
		currentRange.length = endOffset - currentRange.location;
	}
	
	return [muArray autorelease];
}

+ (NSArray*)rangesOfStrings:(NSString*)target range:(NSRange)range word:(NSString*)string ignoreCase:(BOOL)ignoreCase
{
	unsigned endOffset = range.location + range.length;
	unsigned mask = 0;
	if(ignoreCase){ mask |= NSCaseInsensitiveSearch; }
	
	NSRange currentRange = range;
	if(currentRange.location == NSNotFound){ currentRange = NSMakeRange(0, [target length]); }
	
	NSRange wordRange;
	NSMutableArray* muArray = [[NSMutableArray alloc] initWithCapacity:100];
	
	while(YES){
		wordRange = [target rangeOfString:string options:mask range:currentRange];
		if(wordRange.location == NSNotFound){ break; }
		[muArray addObject:[KRangeObj rangeObjWithRange:wordRange]];
		
		currentRange.location = wordRange.location + wordRange.length;
		currentRange.length = endOffset - currentRange.location;
	}
	
	return [muArray autorelease];
}

+ (NSArray*)rangesOfLines:(NSString*)target range:(NSRange)range
{
	NSRange currentRange = NSMakeRange(range.location, 0);
	unsigned endOffset = NSMaxRange(range);
	NSMutableArray* muArray = [[NSMutableArray alloc] initWithCapacity:100];
	
	while(YES){
		currentRange = [target lineRangeForRange:currentRange];
		if(NSMaxRange(currentRange) >= endOffset){
			currentRange.length = endOffset - currentRange.location;
		}
		[muArray addObject:[KRangeObj rangeObjWithRange:currentRange]];
				
		if(NSMaxRange(currentRange) == endOffset){ break; }
		currentRange = NSMakeRange(NSMaxRange(currentRange), 0);
	}
	
	return [muArray autorelease];
}

+ (NSArray*)rangesOfLinesWithoutReturns:(NSString*)target range:(NSRange)range
{
	kunistr targetchars = uni_construct_with_string([target substringWithRange:range]);
	
	unsigned retNum = 1;
	unsigned i;
	for(i = 0; i < targetchars.length; i++){
		if(uni_is_return_char(targetchars.string[i])){ retNum++; }
	}
	NSMutableArray* muArray = [NSMutableArray arrayWithCapacity:retNum];
	
	BOOL isInLine = NO;
	NSRange bufRange;
	
	for(i = 0; i < targetchars.length; i++){
		if(uni_is_return_char(targetchars.string[i]) && isInLine){
			isInLine = NO;
			bufRange.length = i - bufRange.location + range.location;
			[muArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
		}else if(!uni_is_return_char(targetchars.string[i]) && !isInLine){
			isInLine = YES;
			bufRange.location = i + range.location;
		}
	}
	if(isInLine){
		bufRange.length = targetchars.length - bufRange.location + range.location;
		[muArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
	}
	
	uni_destruct(targetchars);

	return muArray;
}

+ (NSArray*)rangesOfLines:(NSString*)target range:(NSRange)range startsWith:(NSString*)string ignoreCase:(BOOL)ignoreCase
{
	NSRange currentRange = NSMakeRange(range.location, 0);
	NSString* bufString;
	unsigned endOffset = NSMaxRange(range);
	unsigned mask = 0;
	if(ignoreCase){ mask |= NSCaseInsensitiveSearch; }
	NSMutableArray* muArray = [[NSMutableArray alloc] initWithCapacity:100];
	
	while(YES){
		currentRange = [target lineRangeForRange:currentRange];
		if(NSMaxRange(currentRange) >= endOffset){
			currentRange.length = endOffset - currentRange.location;
		}
		
		bufString = [target substringWithRange:currentRange];
		if([bufString rangeOfString:string options:mask].location == 0){
			[muArray addObject:[KRangeObj rangeObjWithRange:currentRange]];
		}
		
		if(NSMaxRange(currentRange) == endOffset){ break; }
		currentRange = NSMakeRange(NSMaxRange(currentRange), 0);
	}
	
	return [muArray autorelease];
}

/*
+ (NSArray*)rangesOfQuotedStrings:(NSString*)target range:(NSRange)range
{
	unsigned subLength = range.length;
	unichar* subChars = (unichar*)malloc(subLength * sizeof(unichar));
	[target getCharacters:subChars range:range];
	unsigned i;
	BOOL isInDoubleQuote = NO;
	BOOL isInSingleQuote = NO;
	unichar doubleQuote, singleQuote, backSlash;
	[@"\"" getCharacters:&doubleQuote];
	[@"'" getCharacters:&singleQuote];
	[@"\\" getCharacters:&backSlash];
	NSRange doubleQuotedRange = NSMakeRange(0, 0);
	NSRange singleQuotedRange = NSMakeRange(0, 0);
	NSMutableArray* quotedRangeArray = [NSMutableArray arrayWithCapacity:100];
	
	for(i = 0; i < subLength; i++){
		if(subChars[i] == backSlash){
			i++;
		}else if(subChars[i] == doubleQuote){
			if(isInDoubleQuote){
				doubleQuotedRange.length = i - doubleQuotedRange.location + range.location + 1;
				[quotedRangeArray addObject:[KRangeObj rangeObjWithRange:doubleQuotedRange]];
				isInDoubleQuote = NO;
			}else{
				doubleQuotedRange.location = i + range.location;
				isInDoubleQuote = YES;
				isInSingleQuote = NO;
			}
		}else if((subChars[i] == singleQuote) && (!isInDoubleQuote)){
			if(isInSingleQuote){
				singleQuotedRange.length = i - singleQuotedRange.location + range.location + 1;
				[quotedRangeArray addObject:[KRangeObj rangeObjWithRange:singleQuotedRange]];
				isInSingleQuote = NO;
			}else{
				singleQuotedRange.location = i + range.location;
				isInSingleQuote = YES;
			}
		}
	}
	
	free(subChars);
	
	return quotedRangeArray;
}
*/

+ (NSDictionary*)rangesOfQuotedAndCommentedString:(NSString*)target comment:(NSString*)comment range:(NSRange)range
{	
	// "single_quote", "double_quote", "comment" array.
	// If comment is nil, exist of "comment" object will be undefined.
	
	kunistr subchars = uni_construct_with_string([target substringWithRange:range]);
	kunistr commentchars = uni_construct_with_string(comment);
	
	unsigned i;
	BOOL isInDoubleQuote = NO;
	BOOL isInSingleQuote = NO;
	unichar doubleQuote, singleQuote, backSlash;
	[@"\"" getCharacters:&doubleQuote];
	[@"'" getCharacters:&singleQuote];
	[@"\\" getCharacters:&backSlash];
	
	NSRange doubleQuotedRange, singleQuotedRange, commentRange;
	NSMutableArray* doubleQuotedRangeArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* singleQuotedRangeArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* commentRangeArray = [NSMutableArray arrayWithCapacity:100];
	
	for(i = 0; i < subchars.length; i++){
		if(subchars.string[i] == backSlash){
			i++;
		}else if(subchars.string[i] == doubleQuote){
			if(isInDoubleQuote){
				doubleQuotedRange.length = i - doubleQuotedRange.location + range.location + 1;
				[doubleQuotedRangeArray addObject:[KRangeObj rangeObjWithRange:doubleQuotedRange]];
				isInDoubleQuote = NO;
			}else{
				doubleQuotedRange.location = i + range.location;
				isInDoubleQuote = YES;
				isInSingleQuote = NO;
			}
		}else if((subchars.string[i] == singleQuote) && (!isInDoubleQuote)){
			if(isInSingleQuote){
				singleQuotedRange.length = i - singleQuotedRange.location + range.location + 1;
				[singleQuotedRangeArray addObject:[KRangeObj rangeObjWithRange:singleQuotedRange]];
				isInSingleQuote = NO;
			}else{
				singleQuotedRange.location = i + range.location;
				isInSingleQuote = YES;
			}
		}else if(comment && (i < subchars.length - commentchars.length)
					&& (!isInSingleQuote && !isInDoubleQuote)
					&& (unistrncmp(subchars.string + i, commentchars.string, commentchars.length) == 0)){
			commentRange.location = i + range.location;
			while((i < subchars.length - commentchars.length) 
					&& (subchars.string[i] != NSCarriageReturnCharacter && subchars.string[i] != NSNewlineCharacter)){
				i++;
			}
			commentRange.length = i - commentRange.location + range.location + 1;
			[commentRangeArray addObject:[KRangeObj rangeObjWithRange:commentRange]];
		}
	}
	
	uni_destruct(subchars);
	uni_destruct(commentchars);
	
	return [NSDictionary dictionaryWithObjectsAndKeys:
				doubleQuotedRangeArray, @"double_quote",
				singleQuotedRangeArray, @"single_quote",
				commentRangeArray, @"comment", nil];
}


+ (NSArray*)rangesOfMultiCommentString:(NSString*)target head:(NSString*)head tail:(NSString*)tail range:(NSRange)range
{
	return [KStringUtils rangesOfMultiCommentString:target head:head tail:tail range:range ignoreQuotes:NO];
}

+ (NSArray*)rangesOfMultiCommentString:(NSString*)target head:(NSString*)head tail:(NSString*)tail range:(NSRange)range ignoreQuotes:(BOOL)ignoreQuotes
{
	kunistr substr= uni_construct_with_string([target substringWithRange:range]);
	kunistr headstr = uni_construct_with_string(head);
	kunistr tailstr = uni_construct_with_string(tail);
	unichar singleQuote, doubleQuote;
	[@"'" getCharacters:&singleQuote];
	[@"\"" getCharacters:&doubleQuote];
	
	NSRange bufRange;
	unsigned i;
	NSMutableArray* commentArray = [NSMutableArray arrayWithCapacity:100];
	
	BOOL isInComment = NO;
	//BOOL isInQuote = NO;
	for(i = 0; i < substr.length; i++){
		if(!ignoreQuotes){
			if(substr.string[i] == doubleQuote){
				i++;
				while(i < substr.length && substr.string[i] != doubleQuote){ i++; }
				i++;
			}else if(substr.string[i] == singleQuote){
				i++;
				while(i < substr.length && substr.string[i] != singleQuote){ i++; }
				i++;
			}
		}
		if(i < substr.length - headstr.length && !isInComment
				&& !unistrncmp(substr.string + i, headstr.string, headstr.length)){
			isInComment = YES;
			bufRange.location = i + range.location;
			i += headstr.length - 1;
		}else if(i < substr.length - tailstr.length && isInComment
				&& !unistrncmp(substr.string + i, tailstr.string, tailstr.length)){
			bufRange.length = i - bufRange.location + range.location + tailstr.length;
			[commentArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			isInComment = NO;
			i += tailstr.length - 1;
		}
	}
	
	uni_destruct(substr);
	uni_destruct(headstr);
	uni_destruct(tailstr);
	
	return commentArray;
}


+ (unsigned)countMultiCommentString:(NSString*)target head:(NSString*)head tail:(NSString*)tail range:(NSRange)range
{
	return [KStringUtils countMultiCommentString:target head:head tail:tail range:range ignoreQuotes:NO];
}

+ (unsigned)countMultiCommentString:(NSString*)target head:(NSString*)head tail:(NSString*)tail range:(NSRange)range ignoreQuotes:(BOOL)ignoreQuotes
{
	kunistr substr= uni_construct_with_string([target substringWithRange:range]);
	kunistr headstr = uni_construct_with_string(head);
	kunistr tailstr = uni_construct_with_string(tail);
	unichar singleQuote, doubleQuote;
	[@"'" getCharacters:&singleQuote];
	[@"\"" getCharacters:&doubleQuote];
	
	unsigned i;
	unsigned count = 0;
	
	BOOL isInComment = NO;
	for(i = 0; i < substr.length; i++){
		if(!ignoreQuotes){
			if(substr.string[i] == doubleQuote){
				i++;
				while(i < substr.length && substr.string[i] != doubleQuote){ i++; }
				i++;
			}else if(substr.string[i] == singleQuote){
				i++;
				while(i < substr.length && substr.string[i] != singleQuote){ i++; }
				i++;
			}
		}
		if(i < substr.length - headstr.length && !isInComment
				&& !unistrncmp(substr.string + i, headstr.string, headstr.length)){
			isInComment = YES;
			i += headstr.length - 1;
		}else if(i < substr.length - tailstr.length && isInComment
				&& !unistrncmp(substr.string + i, tailstr.string, tailstr.length)){
			count++;
			isInComment = NO;
			i += tailstr.length - 1;
		}
	}
	
	uni_destruct(substr);
	uni_destruct(headstr);
	uni_destruct(tailstr);
	
	return count;
}

+ (NSDictionary*)rangesOfColorSyntaxWith:(NSString*)target
							range:(NSRange)validatedRange
							multiCommentHead:(NSString*)head // nil ok.
							multiCommentTail:(NSString*)tail // nil ok. if head is nil, ignored.
							singleComment:(NSString*)single // nil ok.
							keywords:(NSArray*)keywords // nil ok. the array of NSString instances.
							variableHead:(NSString*)variableHead // nil ok. head characters as @"$@"
							tokenChars:(NSString*)tokenChars // nil ok. the token characters of keywords and variables.
{
	kunistr targetstr = uni_construct_with_string(target);
	kunistr headstr = uni_construct_with_string(head);
	kunistr tailstr = uni_construct_with_string(tail);
	kunistr singlestr = uni_construct_with_string(single);
	kunistr variableheadstr = uni_construct_with_string(variableHead);
	kunistr tokenstr;
	if(tokenChars){
		tokenstr = uni_construct_with_string(tokenChars);
	}else{
		tokenstr = uni_construct_with_string(KTokenCharacters);
	}
	kunistr* keyarray = uni_construct_with_array(keywords);
	
	kunistr bracket_list_begin = uni_construct_with_string(@"([{<");
	kunistr bracket_list_end = uni_construct_with_string(@")]}>");
	kunistr indchars = uni_construct_with_string(@"QqxrwWs");
	kunistr azAZ01 = uni_construct_with_string(@"0123456789abcdefghijklmmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
	
	unichar singleQuote, doubleQuote, backSlash, slash, cent, space;
	[@"'" getCharacters:&singleQuote];
	[@"\"" getCharacters:&doubleQuote];
	[@"\\" getCharacters:&backSlash];
	[@"/" getCharacters:&slash];
	[@"%" getCharacters:&cent];
	[@" " getCharacters:&space];
	
	unsigned startOffset = validatedRange.location;
	unsigned endOffset = NSMaxRange(validatedRange);
	NSRange bufRange = NSMakeRange(startOffset, 0);
	NSMutableArray* doubleQuoteArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* singleQuoteArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* singleCommentArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* multiCommentArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* keywordArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* variableArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* othersArray = [NSMutableArray arrayWithCapacity:500];
	unsigned i;
	
	for(i = startOffset; i < endOffset; i++){
		if(targetstr.string[i] == backSlash){
			i++;
		}else if(targetstr.string[i] == doubleQuote){ // DOUBLE QUOTES
			if(bufRange.location != i){ [othersArray addObject:[KRangeObj rangeObjWithLocation:bufRange.location andLength:(i - bufRange.location)]]; }
			bufRange.location = i;
			if(i != endOffset - 1){
				i++;
				while(i < endOffset - 1){
					if(targetstr.string[i] == doubleQuote){ break; }
					if(targetstr.string[i] == backSlash && i < endOffset - 2){ i++; }
					i++;
				}
			}
			bufRange.length = i - bufRange.location + 1;
			[doubleQuoteArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			bufRange.location = i + 1;
			
		}else if(targetstr.string[i] == singleQuote){ //SINGLE QUOTE
			if(bufRange.location != i){ [othersArray addObject:[KRangeObj rangeObjWithLocation:bufRange.location andLength:(i - bufRange.location)]]; }
			bufRange.location = i;
			if(i != endOffset - 1){
				i++;
				while(i < endOffset - 1){
					if(targetstr.string[i] == singleQuote){ break; }
					if(targetstr.string[i] == backSlash && i < endOffset - 2){ i++; }
					i++;
				}
			}
			bufRange.length = i - bufRange.location + 1;
			[singleQuoteArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			bufRange.location = i + 1;
		
		}else if(targetstr.string[i] == cent && endOffset - 2){ // %
			if(targetstr.string[i + 1] == space){ continue; }
			i++;
			if(uni_does_contain_char(indchars, targetstr.string[i])){ i++; }
			if(targetstr.string[i] == space || uni_does_contain_char(azAZ01, targetstr.string[i])){ continue; }
			unichar spacer_begin, spacer_end;
			spacer_begin = spacer_end = targetstr.string[i];
			int bracket_offset;
			if((bracket_offset = uni_offset_of_char(bracket_list_begin, spacer_begin)) >= 0){ // bracket.
				spacer_end = bracket_list_end.string[bracket_offset];
				unsigned ind = 1;
				i++;
				while(i < endOffset && !uni_is_return_char(targetstr.string[i])){
					if(targetstr.string[i] == backSlash){ i++; continue; }
					
					if(targetstr.string[i] == spacer_begin){
						ind++;
					}else if(targetstr.string[i] == spacer_end){
						ind--;
					}
					if(ind == 0){ break; }
					i++;
				}
				
			}else{ // not bracket.
				//NSLog([NSString stringWithFormat:@"off = %d, start = %d, end = %d", bracket_offset, spacer_begin, spacer_end]);
				i++;
				while(i < endOffset && !uni_is_return_char(targetstr.string[i]) && targetstr.string[i] != spacer_end){
					if(i < endOffset - 1 && targetstr.string[i] == backSlash){ i++; }
					i++;
				}
			}
			
		}else if(singlestr.length && i < endOffset - singlestr.length
					&& !unistrncmp(targetstr.string + i, singlestr.string, singlestr.length)){ // SINGLE COMMENT
			if(bufRange.location != i){ [othersArray addObject:[KRangeObj rangeObjWithLocation:bufRange.location andLength:(i - bufRange.location)]]; }
			bufRange.location = i;
			i += singlestr.length;
			while(i < endOffset && !uni_is_return_char(targetstr.string[i])){ i++; }
			bufRange.length = i - bufRange.location;
			[singleCommentArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			bufRange.location = i + 1;
			
		}else if(headstr.length && i < endOffset - headstr.length
					&& !unistrncmp(targetstr.string + i, headstr.string, headstr.length)){ // MULTI-COMMENT
			if(bufRange.location != i){ [othersArray addObject:[KRangeObj rangeObjWithLocation:bufRange.location andLength:(i - bufRange.location)]]; }
			bufRange.location = i;
			i += headstr.length;
			while(i < endOffset - tailstr.length && unistrncmp(targetstr.string + i, tailstr.string, tailstr.length)){ i++; }
			bufRange.length = i - bufRange.location + tailstr.length;
			[multiCommentArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			i += tailstr.length - 1;
			bufRange.location = i + 1;
			
			
		}else if(targetstr.string[i] == slash && i < endOffset - 1){ // slash
			unsigned offset;
			for(offset = 1; offset + i < endOffset; offset++){
				if(targetstr.string[offset + i] == backSlash && offset + i < endOffset - 1){
					offset++;
					continue;
				}else if(targetstr.string[offset + i] == slash){
					i += offset;
					break;
				}else if(uni_is_return_char(targetstr.string[offset + i])){
					break;
				}
			}
			
		}else if(uni_does_contain_char(variableheadstr, targetstr.string[i])){ // VARIABLE
			if(bufRange.location != i){ [othersArray addObject:[KRangeObj rangeObjWithLocation:bufRange.location andLength:(i - bufRange.location)]]; }
			bufRange.location = i;
			if(i < endOffset - 2){
				i++;
				while(i < endOffset && uni_does_contain_char(tokenstr, targetstr.string[i])){ i++; }
			}
			if(i == bufRange.location){ break; }
			bufRange.length = i - bufRange.location;
			[variableArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			bufRange.location = i;
			i--;
		
		}else if(keyarray && uni_does_contain_char(tokenstr, targetstr.string[i])){  // KEYWORDS
			NSRange tokenRange;
			tokenRange.location = i;
			i++;
			while(i < endOffset && uni_does_contain_char(tokenstr, targetstr.string[i])){ i++; }
			tokenRange.length = i - tokenRange.location;
			
			unsigned j = 0;
			while(keyarray[j].length){
				if(tokenRange.length == keyarray[j].length && !unistrncmp(targetstr.string + tokenRange.location, keyarray[j].string, keyarray[j].length)){
					if(bufRange.location != tokenRange.location){ [othersArray addObject:[KRangeObj rangeObjWithLocation:bufRange.location andLength:(tokenRange.location - bufRange.location)]]; }
					bufRange.location = i;
					[keywordArray addObject:[KRangeObj rangeObjWithRange:tokenRange]];
					break;
				}
				j++;
			}
			i--;
		}
	}
	if(i <= endOffset && bufRange.location != i){
		[othersArray addObject:[KRangeObj rangeObjWithLocation:bufRange.location andLength:(i - bufRange.location)]];
	}

	
	uni_destruct(tokenstr);
	uni_destruct(variableheadstr);
	uni_destruct(singlestr);
	uni_destruct(tailstr);
	uni_destruct(headstr);
	uni_destruct(targetstr);
	uni_destruct(bracket_list_begin);
	uni_destruct(bracket_list_end);
	uni_destruct(indchars);
	uni_destruct(azAZ01);
	
	uni_destruct_array(keyarray);
	
	NSDictionary* dict = [NSDictionary dictionaryWithObjectsAndKeys:
							doubleQuoteArray, @"double_quote",
							singleQuoteArray, @"single_quote",
							singleCommentArray, @"single_comment",
							multiCommentArray, @"multi_comment",
							keywordArray, @"keywords",
							variableArray, @"variables",
							othersArray, @"others", 
							nil];
	
	return dict;
}

+ (NSDictionary*)rangesOfHMLSyntaxWith:(NSString*)target
							range:(NSRange)validatedRange
							keywords:(NSArray*)keywords // not supported.
{
	kunistr targetstr = uni_construct_with_string(target);
	kunistr commenthead = uni_construct_with_string(@"<!--");
	kunistr commenttail = uni_construct_with_string(@"-->");
	unichar tagHead, tagTail, amp, semicolon, doubleQuote, singleQuote;
	[@"<" getCharacters:&tagHead];
	[@">" getCharacters:&tagTail];
	[@"&" getCharacters:&amp];
	[@";" getCharacters:&semicolon];
	[@"\"" getCharacters:&doubleQuote];
	[@"\'" getCharacters:&singleQuote];
	NSMutableArray* tagArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* commentArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* othersArray = [NSMutableArray arrayWithCapacity:500];
	NSMutableArray* keywordArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* singleQuoteArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* doubleQuoteArray = [NSMutableArray arrayWithCapacity:100];
	NSMutableArray* refArray = [NSMutableArray arrayWithCapacity:100];
	unsigned startOffset = validatedRange.location;
	unsigned endOffset = NSMaxRange(validatedRange);
	unsigned i;
	NSRange bufRange = NSMakeRange(startOffset, 0);
	
	for(i = startOffset; i < endOffset; i++){
		if(i < endOffset - commenthead.length
				&& !unistrncmp(targetstr.string + i, commenthead.string, commenthead.length)){ // COMMENT
			if(bufRange.location != i){
				bufRange.length = i - bufRange.location;
				[othersArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			}
			bufRange.location = i;
			while(i < endOffset - commenttail.length
					&& unistrncmp(targetstr.string + i, commenttail.string, commenttail.length)){
				i++;
			}
			bufRange.length = i - bufRange.location + commenttail.length;
			[commentArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			i += commenttail.length - 1;
			bufRange.location = i + 1;
			
		}else if(targetstr.string[i] == tagHead){ // TAG
			if(bufRange.location != i){
				bufRange.length = i - bufRange.location;
				[othersArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			}
			bufRange.location = i;
			while(i < endOffset - 1 && targetstr.string[i] != tagTail){
				if(targetstr.string[i] == doubleQuote){ // DOUBLE QUOTE
					if(bufRange.location != i){
						bufRange.length = i - bufRange.location;
						[tagArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
					}
					bufRange.location = i;
					i++;
					while(i < endOffset - 1 && targetstr.string[i] != doubleQuote){ i++; }
					bufRange.length = i - bufRange.location + 1;
					[doubleQuoteArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
					bufRange.location = i + 1;
				}else if(targetstr.string[i] == singleQuote){ // SINGLE QUOTE
					if(bufRange.location != i){
						bufRange.length = i - bufRange.location;
						[tagArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
					}
					bufRange.location = i;
					i++;
					while(i < endOffset - 1 && targetstr.string[i] != singleQuote){ i++; }
					bufRange.length = i - bufRange.location + 1;
					[singleQuoteArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
					bufRange.location = i + 1;
				}
				i++;
			}
			if(i < endOffset){
				bufRange.length = i - bufRange.location + 1;
				[tagArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
				bufRange.location = i + 1;
			}
			
		}else if(targetstr.string[i] == amp){ // REFERENCE
			if(bufRange.location != i){
				bufRange.length = i - bufRange.location;
				[othersArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			}
			bufRange.location = i;
			if(i < endOffset - 1){
				i++;
				while(i < endOffset - 1 && targetstr.string[i] != semicolon){ i++; }
			}
			bufRange.length = i - bufRange.location + 1;
			[refArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
			bufRange.location = i + 1;
		}
	}
	if(i <= endOffset && bufRange.location != i){
		bufRange.length = i - bufRange.location;
		[othersArray addObject:[KRangeObj rangeObjWithRange:bufRange]];
	}
	
	uni_destruct(commenthead);
	uni_destruct(commenttail);
	uni_destruct(targetstr);
	
	NSDictionary* dict = [NSDictionary dictionaryWithObjectsAndKeys:
							tagArray, @"tags",
							commentArray, @"comments",
							othersArray, @"others",
							keywordArray, @"keywords",
							singleQuoteArray, @"single_quote",
							doubleQuoteArray, @"double_quote",
							refArray, @"refs", nil];
	
	return dict;
}


+ (NSString*)replaceString:(NSString*)string search:(NSString*)search replace:(NSString*)replace
{
	NSArray* array = [string componentsSeparatedByString:search];
	return [array componentsJoinedByString:replace];
}

+ (NSString*)wrapString:(NSString*)string width:(unsigned)width
{
	// return code == lf.
	kunistr targetstr = uni_construct_with_string(string);
	kunistr outstr = uni_construct_with_size([string length] * 2);
	unsigned i;
	unsigned count = 0;
	
	for(i = 0; i < [string length]; i++){
		if(uni_is_return_char(targetstr.string[i])){
			count = 0;
			uni_append_char(&outstr, NSNewlineCharacter);
			continue;
		}
		if(targetstr.string[i] > 0xFF){ count++; }
		count++;
		
		if(count > width){
			uni_append_char(&outstr, NSNewlineCharacter);
			count = 2;
		}
		uni_append_char(&outstr, targetstr.string[i]);
		
		if(count == width && i < targetstr.length - 1 && !uni_is_return_char(targetstr.string[i + 1])){
			uni_append_char(&outstr, NSNewlineCharacter);
			count = 0;
		}
	}
	
	NSString* outString = [NSString stringWithCharacters:outstr.string length:outstr.length];
	uni_destruct(targetstr);
	uni_destruct(outstr);
	
	return outString;
}



// case-sensitive search.
+ (NSString*)convertCStyleToString:(NSString*)string
{
	NSString* aString = string;
	aString = [self replaceString:aString search:@"\\r" replace:@"\n"];
	aString = [self replaceString:aString search:@"\\n" replace:@"\n"];
	aString = [self replaceString:aString search:@"\\t" replace:@"\t"];
	aString = [self replaceString:aString search:@"\\\"" replace:@"\""];
	aString = [self replaceString:aString search:@"\\\\" replace:@"\\"];
	return aString;
	// for UTF-16 only.
}

+ (unsigned)unsignedIntFromString:(NSString*)string
{
	unsigned length = [string length];
	if(([[KStringUtils rangesOfTokens:string withTokenCharacters:KHexIntCharacters] count] != 1)
			|| (length > 4) || (length == 0)){ return NSNotFound; }
	
	unsigned i;
	unsigned num = 0x00;
	unsigned bufNum;
	NSString* bufString;
	
	for(i = 0; i < length; i++){
		bufString = [string substringWithRange:NSMakeRange(length - i - 1, 1)];
		bufNum = [bufString intValue];
		
		if(bufNum){ num |= (bufNum << 4 * i); }
		else if([[bufString uppercaseString] isEqualToString:@"A"]){ num |= (0x0A << 4 * i); }
		else if([[bufString uppercaseString] isEqualToString:@"B"]){ num |= (0x0B << 4 * i); }
		else if([[bufString uppercaseString] isEqualToString:@"C"]){ num |= (0x0C << 4 * i); }
		else if([[bufString uppercaseString] isEqualToString:@"D"]){ num |= (0x0D << 4 * i); }
		else if([[bufString uppercaseString] isEqualToString:@"E"]){ num |= (0x0E << 4 * i); }
		else if([[bufString uppercaseString] isEqualToString:@"F"]){ num |= (0x0F << 4 * i); }
	}
	return num;
}

+ (NSString*)stringFromUnsignedInt:(unsigned)num length:(unsigned)length
{
	if((length <= 0) || (length > 8)){ return nil; }
	
	NSString* numString = [NSString stringWithFormat:@"%X",num];
	NSMutableString* muString = [NSMutableString stringWithCapacity:10];
	unsigned i;
	for(i = 0; i < length - [numString length]; i++){
		[muString appendString:@"0"];
	}
	return [muString stringByAppendingString:numString];
}

+ (NSString*)stringFromColor:(NSColor*)color
{
	float red, green, blue, alpha;
	[color getRed:&red green:&green blue:&blue alpha:&alpha];
	
	return [NSString stringWithFormat:@"#%@%@%@",
				[KStringUtils stringFromUnsignedInt:(red * 255.0) length:2],
				[KStringUtils stringFromUnsignedInt:(green * 255.0) length:2],
				[KStringUtils stringFromUnsignedInt:(blue * 255.0) length:2]
			];
}

+ (NSColor*)colorFromString:(NSString*)string
{
	NSArray* array = [KStringUtils rangesOfTokens:string withTokenCharacters:KHexIntCharacters];
	if([array count] != 1){ return nil; }
	
	NSString* numString = [array objectAtIndex:0];
	if(![KStringUtils string:string startsWith:@"#"] 
			|| ([string length] != 7)
			|| ([numString length] != 6)){
		return nil;
	}
	
	float red = ((float)[KStringUtils unsignedIntFromString:[string substringWithRange:NSMakeRange(1, 2)]]) / 255.0;
	float green = ((float)[KStringUtils unsignedIntFromString:[string substringWithRange:NSMakeRange(3, 2)]]) / 255.0;
	float blue = ((float)[KStringUtils unsignedIntFromString:[string substringWithRange:NSMakeRange(5, 2)]]) / 255.0;
		
	return [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0];
}

+ (NSString*)trimSpaces:(NSString*)string
{
	NSArray* array = [KStringUtils string:string componentsSeparatedByCharacters:KSpaceCharacters];
	return [array componentsJoinedByString:@" "];
}

//for KEdit tag-file
+ (NSDictionary*)resolveCommand:(NSString*)command
{
	NSString* string;
	NSArray* array = [command componentsSeparatedByString:@"\""];
	if([array count] < 3){
		string = [NSString stringWithString:@""];
	}else{
		NSMutableArray* mutableArray = [NSMutableArray arrayWithCapacity:20];
		unsigned i;
		for(i = 1; i < [array count] - 1; i++){
			[mutableArray addObject:[array objectAtIndex:i]];
		}
		string = [mutableArray componentsJoinedByString:@"\""];
		//NSLog([NSString stringWithFormat:@"string = %@", string]);
		string = [self convertCStyleToString:string];
	}
	array = [self string:command componentsSeparatedByCharacters:@" \t"];
	
	NSDictionary* dict = [[NSDictionary alloc] initWithObjectsAndKeys:
							[array objectAtIndex:0], @"command",
							string, @"string"
							,nil];
	

	return [dict autorelease];
}

// KEdit settings.txt files.
+ (NSDictionary*)settingsFromString:(NSString*)string
{
	NSArray* lineArray = [KStringUtils string:string componentsSeparatedByCharacters:@"\r\n"];
	NSArray* bufArray;
	NSString* bufString;
	NSMutableDictionary* muDict = [NSMutableDictionary dictionaryWithCapacity:20];
	unsigned i;
	
	for(i = 0; i < [lineArray count]; i++){
		bufString = [lineArray objectAtIndex:i];
		if([KStringUtils string:[KStringUtils trimSpaces:bufString] startsWith:@"//"]){ continue; }
		bufArray = [bufString componentsSeparatedByString:@"="];
		if([bufArray count] != 2){ continue; }
		
		[muDict setObject:[KStringUtils trimSpaces:[bufArray objectAtIndex:1]] forKey:[KStringUtils trimSpaces:[bufArray objectAtIndex:0]]];
	}
	return muDict;
}

@end


















