//
//  KTextUtils.m
//  KEdit
//
//  Created by Karino, Masatsugu on Thu Feb 20 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "KTextUtils.h"
#import "KStringUtils.h"

@implementation KTextUtils

+ (KReturnCodeType) detectReturnCodeType:(NSData*) data
{
	const char* text = [data bytes];
	
	while(*(text++) != '\0'){
		if(*text == LF_char){
			return KLF_type;
		}else if(*text == CR_char){
			if(*(++text) == LF_char){
				return KCRLF_type;
			}else{
				return KCR_type;
			}
		}
	}
	return KLF_type; //default is LF(for MacOSX)
}

+ (NSData*) convertReturnCode:(NSData*) data from:(KReturnCodeType) fromType to:(KReturnCodeType) toType
{
	if(fromType == toType){ return [NSData dataWithData:data]; }
	
	NSData* newData;
	
	if((fromType == KCR_type && toType == KLF_type)
		|| (fromType == KLF_type && toType == KCR_type)){
		char fromChar, toChar;
		
		if(fromType == KCR_type){ fromChar = CR_char; toChar = LF_char; }
		else{ fromChar = LF_char; toChar = CR_char; }
		
		NSMutableData* bufData = [NSMutableData dataWithData:data];
		char* text = [bufData mutableBytes];
		while(*text != '\0'){
			if(*text == fromChar){ *text = toChar; }
			text++;
		}
		newData = [NSData dataWithData:bufData];
	
	}else if(fromType == KCRLF_type){
		char removeChar;
		if(toType == KCR_type){ removeChar = LF_char; }
		else{ removeChar = CR_char; }
		
		NSMutableData* bufData = [NSMutableData dataWithData:data];
		char* text = [bufData mutableBytes];
		long currentOffset = 0;
		long copiedOffset = 0;
		for(;*(text + currentOffset) != '\0'; currentOffset++){
			if(*(text + currentOffset) != removeChar){
				*(text + copiedOffset) = *(text + currentOffset);
				copiedOffset++;
			}
		}
		*(text + copiedOffset) = '\0';
				
		newData = [NSData dataWithData:bufData];
		
	}else if(toType == KCRLF_type){
		char fromChar;
		char* dataString = (char*)[data bytes];
		long dataLength = [data length];
		
		if(fromType == KCR_type){ fromChar = CR_char; }
		else{ fromChar = LF_char; }
		
		long offset = 0;
		long returnCount = 0;
		for(; *(dataString + offset) != '\0'; offset++){
			if(*(dataString + offset) == fromChar){ returnCount++; }
		}
		
		NSMutableData* bufData = [NSMutableData dataWithLength:(dataLength + returnCount)];
		char* text = [bufData mutableBytes];
		while(*dataString != '\0'){
			if(*dataString == fromChar){
				*text++ = CR_char;
				*text++ = LF_char;
			}else{
				*text++ = *dataString;
			}
			dataString++;
		}
		*text = '\0';
		
		newData = [NSData dataWithData:bufData];
	}
	
	return newData;
}


// Character-Code Detector
+ (NSStringEncoding) detectCharacterCodeType:(NSData*) data
{
	unsigned char	c = 1; // unsigned char is required.
	const char* text = [data bytes];
	const char* text2 = [data bytes];
	KDetectedCharacterCodeType	aCodeType = ASCII_type;
	
	//for UTF-8/16 only. It's not a smart way.
	
	int		codenum, i;
	long	len = strlen(text);
	bool	isASCII = true;
	
	char aUTF16Head[3] = { 0xFE, 0xFF, '\0' };
	if((len >= 2) && !strncmp(text, aUTF16Head, 2)){
		aCodeType = UTF16_type;
	}
	
	
	//int testCount = 0;
	while ((aCodeType == EUCORSJIS_type || aCodeType == ASCII_type) && c != '\0'){
		//testCount++;
		//if(testCount < 20){ NSLog([NSString stringWithFormat:@"c = %hu, 0x%x", c, c]); }
	
		if((c = *text++) != '\0'){
			if(c == ESC_char){
				c = *text++;
				if(c == '$'){
					c = *text++;
					if(c == 'B')
						aCodeType = NEW_type;
					else if(c == '@')
						aCodeType = OLD_type;
				}
				else if(c == 'K')
					aCodeType = NEC_type;
			}
			else if((c >= 129 && c <= 141) || (c >= 143 && c <= 159)){
				aCodeType = SJIS_type;
			}
			else if(c == SS2_char){
				c = *text++;
				if((c >= 64 && c <= 126) || (c >= 128 && c <= 160) || (c >= 224 && c <= 252))
					aCodeType = SJIS_type;
				else if(c >= 161 && c <= 223)
					aCodeType = EUCORSJIS_type;
			}
			else if(c >= 161 && c <= 223){
				c = *text++;
				if(c >= 240 && c <= 254)
					aCodeType = EUC_type;
				else if(c >= 161 && c <= 223)
					aCodeType = EUCORSJIS_type;
				else if(c >= 224 && c <= 239){
					aCodeType = EUCORSJIS_type;
					while (c >= 64 && c != '\0' && aCodeType == EUCORSJIS_type){
						if(c >= 129){
							if(c <= 141 || (c >= 143 && c <= 159))
								aCodeType = SJIS_type;
							else if(c >= 253 && c <= 254)
								aCodeType = EUC_type;
						}
						c = *text++;
					}
				}
				else if(c <= 159)
					aCodeType = SJIS_type;
			}
			else if(c >= 240 && c <= 254)
				aCodeType = EUC_type;
			else if(c >= 224 && c <= 239){
				c = *text++;
				if((c >= 64 && c <= 126) || (c >= 128 && c <= 160))
					aCodeType = SJIS_type;
				else if(c >= 253 && c <= 254)
					aCodeType = EUC_type;
				else if(c >= 161 && c <= 252)
					aCodeType = EUCORSJIS_type;
			}
		}
	}
	if(/*aCodeType == ASCII_type || */(aCodeType != UTF16_type) || (aCodeType != SJIS_type)){
		for(i = 0; i < len; i++){
			if((codenum = [KTextUtils getCodeLength:(text2 + i)]) > 1){
				if([KTextUtils getCodeLength:(text2 + i + 1)] != 0
					|| (codenum == 3 && [KTextUtils getCodeLength:(text2 + i + 2)] != 0)){
					break;
				}
				isASCII = false;
			}
			if(i == len - 1 && !isASCII){
				aCodeType = UTF8_type;
			}
		}
	}
	//NSLog([NSString stringWithFormat:@"code = %d, testcount = %d", aCodeType, testCount]);
	
	if(aCodeType == ASCII_type || aCodeType == UTF8_type){
		return NSUTF8StringEncoding;
	}else if(aCodeType == NEW_type || aCodeType == OLD_type){
		return NSISO2022JPStringEncoding;
	}else if(aCodeType == SJIS_type){
		return NSShiftJISStringEncoding;
	}else if(aCodeType == EUC_type || aCodeType == EUCORSJIS_type){
		return NSJapaneseEUCStringEncoding;
	}else if(aCodeType == UTF16_type){
		return NSUnicodeStringEncoding;
	}
	
	NSLog(@"KTextUtils detectCodeType: an error has occured.");
	return NSShiftJISStringEncoding;
}

+ (int)getCodeLength:(const char*) text
{
	//Now, for 3 bytes.
	if((*text & 0xc0) != 0x80){
		if((*text & 0x80) == 0){
			return 1;
		}else if((*text & 0xe0) == 0xc0){
			return 2;
		}else if((*text & 0xf0) == 0xe0){
			return 3;
		}else{
			return 0;
		}
	}else{
		return 0;
	}
}

@end
