/* KTextView */

#import <Cocoa/Cocoa.h>

typedef enum {
	KSelectBeginingOfInsersionPoint = 1,
	KSelectEndOfInsertionPoint,
	KSelectWholeInsertionPoint,
	KSelectInsideInsertionPoint
} KInsertedSelectionType;

@class KKeyAssignServer;


@interface KTextView : NSTextView
{
	IBOutlet id myDocument;
	
	KKeyAssignServer* mKeyAssignServer;
	NSRange mLastAffectedRange;
}

//Utility methods.
- (void)insertTagsWithPrefix:(NSString*)prefix suffix:(NSString*)suffix;
- (void)insertTagsWithPrefix:(NSString*)prefix suffix:(NSString*)suffix single:(NSString*)single insertion:(KInsertedSelectionType)type;

- (void)selectLineTopOf:(unsigned)lineIndex;
- (void)selectLineOf:(unsigned)lineIndex;

//NSView methods.
- (id)initWithFrame:(NSRect)rect;
- (NSMenu*)menuForEvent:(NSEvent*)event;

// NSText methods.
//- (void)replaceCharactersInRange:(NSRange)aRange withRTF:(NSData*)rtfData;
//- (void)replaceCharactersInRange:(NSRange)aRange withRTFD:(NSData*)rtfdData;
//- (void)replaceCharactersInRange:(NSRange)aRange withString:(NSString*)aString;

//NSTextView methods.
//- (void)insertText:(id)aString;
//- (BOOL)shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString;
- (BOOL)shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString*)replacementString;
- (void)updateFontPanel;

//NSResponder methods.
- (void)keyDown:(NSEvent*)theEvent;

//NSView-NSDraggingDestination protocol methods.
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender;
- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender;
- (BOOL)prepareDragOperation:(id <NSDraggingInfo>)sender;
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender;
- (void)concludeDragOperation:(id <NSDraggingInfo>)sender;

//KTextView accessors.
- (NSRange)lastAffectedRange;

//KTextView internal methods.
- (void)insertTextOfFile:(NSString*)fileName;
- (void)insertRelativePath:(NSString*)fileName;
- (void)insertImageTag:(NSString*)fileName;
- (void)insertURL:(NSString*)urlString;
- (void)insertColorString:(NSColor*)color;

//Action from the color panel.
- (void)changeColor:(id)sender;

//Additional keyassigns.
- (void)moveForwardAndModifySelection_kedit:(id)sender;
- (void)moveBackwardAndModifySelection_kedit:(id)sender;
- (void)moveToBeginningOfParagraphAndModifySelection:(id)sender;
- (void)moveToEndOfParagraphAndModifySelection:(id)sender;

- (void)doSample:(id)sender;


//Actions for MENUs
- (IBAction)insertMenuTag:(id)sender;
- (IBAction)insertMenuURL:(id)sender;

- (IBAction)shiftRight:(id)sender;
- (IBAction)shiftLeft:(id)sender;

- (IBAction)wrapWithReturn:(id)sender;
- (IBAction)removeReturns:(id)sender;

//pass actions.
- (BOOL)isAutoIndented;





@end
