//
//  MyDocument.h
//  KEdit
//
//  Created by Karino, Masatsugu on Tue Feb 18 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//


#import <Cocoa/Cocoa.h>
#import "KTextUtils.h"
#import "KSearchServer.h"
#import "KPrefsServer.h"

//#define KStoreWindowStatusForAttr @"kedit_store_window_status"
//#define KTextSelectedRangeForAttr @"kedit_text_selected_range"

// constants
//float kUnwrappedViewWidth = 5000;

#define KPlainName @"Plain"
#define KHTMLName @"X/HTML"
#define KIndexName @"Indexed"
#define KKEditMarkupName @"k-markup"
#define KCSSName @"CSS"
#define KShellScriptName @"sh-script"
#define KPerlScriptName @"perl"
#define KRubyScriptName @"ruby"
#define KPHPName @"PHP"

@class KColorSyntaxServer;


@interface MyDocument : NSDocument
{
	@private
	IBOutlet id myTextView;
	//IBOutlet id myLineNumberView;
	IBOutlet id myScrollView;
	IBOutlet id myWindow;
	
	KColorSyntaxServer* mColorSyntaxServer;
	
	NSString* mBufString;
	NSRange mSelectedRange;
	
	KDocumentCharacterCodeType 	mDocumentEncoding;
	KDocumentReturnCodeType 	mReturnCodeType;
	
  KDocumentTextFileFormatType mTextType;
	
	NSString* mTextViewFontName;
	float mTextViewFontSize;
	
	BOOL mWordWrap;
	BOOL mAutoIndent;
	BOOL mStoreWindowStatus;
	
	unsigned int mCurrentLineNumber;
}

// from SearchServer
- (void)doSearch:(id)sender direction:(KSearchDirection)direction;
- (void)doReplaceAll:(id)sender;

- (void)dealloc;

//Hook Functions
//NSDocument
- (BOOL)revertToSavedFromFile:(NSString*)fileName ofType:(NSString*)type;
- (BOOL)validateMenuItem:(NSMenuItem*)anItem;
- (NSDictionary*)fileAttributesToWriteToFile:(NSString*)fullDocumentPath ofType:(NSString*)docType saveOperation:(NSSaveOperationType)saveOperationType;
	//- (BOOL)writeToFile:(NSString *)fileName ofType:(NSString *)docType;
//NSTextView
//- (BOOL)textView:(NSTextView*)aTextView shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString;
- (void)textViewDidChangeSelection:(NSNotification *)aNotification;
//NSSText
- (void)textDidChange:(NSNotification*)aNotification;
//NSWindow
- (void)windowDidBecomeMain:(NSNotification *)aNotification;
//NSObject
- (void)dealloc;

//access data-members
- (BOOL)isWordWrapped;
- (void)setWordWrapped:(BOOL)wordwrap;
- (BOOL)isAutoIndented;
- (void)setAutoIndented:(BOOL)autoindent;

- (NSString*)textViewFontName;
- (void)setTextViewFontName:(NSString*)fontName;
- (float)textViewFontSize;
- (void)setTextViewFontSize:(float)fontSize;

- (void)setCurrentLineNumber:(unsigned int)num;
- (unsigned int)currentLineNumber;

- (KDocumentTextFileFormatType)textType;
- (void)setTextType:(KDocumentTextFileFormatType)type;
- (KDocumentCharacterCodeType)documentEncoding;
- (void)setDocumentEncoding:(KDocumentCharacterCodeType)type;
- (KDocumentReturnCodeType)returnCodeType;
- (void)setReturnCodeType:(KDocumentReturnCodeType)type;

- (NSFont*)textViewFont;


//internal use
- (void)setTextTypeWithTypeName:(NSString*)aType;// such as 'text file'; not KTextType.

- (void)showReplacedAllDialog:(unsigned)count;
- (void)showReplacedAllDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;

- (void)adjustDocumentSettings;

@end


@interface MyDocument (Actions)
//(use "charactercodetype" menu for force converting)
- (IBAction)forceToChangeEncodingDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;

//Actions for MENUs
- (IBAction)scrollToSelection:(id)sender;
- (IBAction)toggleWordWrap:(id)sender;
- (IBAction)toggleAutoIndent:(id)sender;
- (IBAction)toggleStoreWindowStatus:(id)sender;

- (IBAction)showGotoLineWindow:(id)sender;

- (IBAction)showPreferenceWindow:(id)sender;
- (IBAction)showColorPanel:(id)sender;
- (IBAction)showFontPanel:(id)sender;

- (IBAction)changeFont:(id)sender;

- (IBAction)redrawColors:(id)sender;
- (IBAction)openWithBrowser:(id)sender;

- (IBAction)executeScript:(id)sender;

- (IBAction)showInformationDialog:(id)sender;
- (IBAction)showInformationDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;

//Actions for ColorPanel
- (IBAction)insertColorString:(id)sender;

//Actions for Search Functions.
- (IBAction)searchShowWindow:(id)sender;
- (IBAction)searchForward:(id)sender;
- (IBAction)searchBackward:(id)sender;
- (IBAction)setSelectionToSearch:(id)sender;
- (IBAction)replace:(id)sender;
- (IBAction)replaceAndFindAgain:(id)sender;

//Actions for BottomView.
- (IBAction)setSelectedRangeOfTextView:(id)sender;

//Actions for GotoLineWindow.
- (IBAction)setCurrentLineOfTextView:(id)sender;

@end


@interface MyDocument (ColorSyntax)

- (void)validateColorSyntax;
- (void)validateColorSyntax:(NSRange)validatedRange;

@end


@interface MyDocument (ScriptingSupport)
- (NSString*)retcode;
- (void)setRetcode:(NSString*)newRetcode;

- (NSString*)syntaxtype;
- (void)setSyntaxtype:(NSString*)newSyntaxType;

- (NSString*)fontname;
- (void)setFontname:(NSString*)newFont;

- (NSNumber*)fontsize;
- (void)setFontsize:(NSNumber*)newSize;

- (NSDictionary*)selectedrange;
- (void)setSelectedrange:(NSDictionary*)newSelectedRange;

- (NSNumber*)wordwrap;
- (void)setWordwrap:(NSNumber*)newWordWrap;

- (NSNumber*)autoindent;
- (void)setAutoindent:(NSNumber*)newAutoIndent;

- (NSTextStorage*)selectedtext;
- (void)setSelectedtext:(NSTextStorage*)newText;

- (NSNumber*)selectionLocation;
- (void)setSelectionLocation:(NSNumber*)newSelectionLocation;
- (NSNumber*)selectionLength;
- (void)setSelectionLength:(NSNumber*)newSelectionLength;

- (NSTextStorage*)text;

// ScriptCommands.
- (NSNumber*)handleReplaceAllCommand:(NSScriptCommand*)command;
- (NSNumber*)handleSearchCommand:(NSScriptCommand*)command;

- (void)handleScrollToSelectionCommand:(NSScriptCommand*)command;

@end








