
#include "kregex.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

kr_range* kr_range_construct(unsigned location, unsigned length)
{
  kr_range* range = (kr_range*)malloc(sizeof(kr_range));
  range->location = location;
  range->length = length;
  return range;
}

void kr_range_destruct(kr_range* range)
{
  free(range);
}

void kr_range_print_to_stream(kr_range* range)
{
  printf("kr_range : locatin = %d, length = %d\n", range->location, range->length);
}

kr_range* kr_range_duplicate(kr_range* range)
{
  return kr_range_construct(range->location, range->length);
}

int kr_range_shift(kr_range* range, unsigned value)
{
  if (value >= range->location){ return FALSE; }
  range->location += value;
  range->length -= value;
  return TRUE;
}



kr_array* kr_array_construct(unsigned count, unsigned block_size)
{
  kr_array* array = (kr_array*)malloc(sizeof(kr_array));
	array->count = count;
	array->_block_size = block_size;
	array->_mem_size = (count / block_size + 1) * block_size;
	array->ptrs = (unsigned*)malloc(array->_mem_size);
	return array;
}

void kr_array_destruct(kr_array* array)
{
  free(array->ptrs);
	free(array);
}

void kr_array_print_to_stream(kr_array* array)
{
  printf("kr_array : count = %d, _mem_size = %d, _block_size = %d\n", array->count, array->_mem_size, array->_block_size);
}

void kr_array_add(kr_array* array, void* ptr)
{
  array->count++;
  if (array->count > array->_mem_size){
	  array->_mem_size = (array->count / array->_block_size + 1) * array->_block_size;
	  array->ptrs = realloc(array->ptrs, array->_mem_size);
	}
	array->ptrs[array->count - 1] = (unsigned*)ptr;
}


/*
kr_string* kr_string_construct(kr_char* data, unsigned count)
{
  return kr_string_construct_with_detail(data, count, TRUE, 0);
}

kr_string* kr_string_construct_with_detail(kr_char* data, unsigned count, int will_allocate_memory, unsigned default_block_size)
{
  kr_string* str = (kr_string*)malloc(sizeof(kr_string));
  str->count = count;
	str->_frozen = FALSE;
  str->_mem_size = sizeof(kr_char) * count;
	str->_will_allocate_memory = will_allocate_memory;
  if (will_allocate_memory){
    str->data = (kr_char*)malloc(str->_mem_size);
    memcpy(str->data, data, str->_mem_size);
  }else{
    str->data = data;
  }
  
  return str;
}
*/

kr_string* kr_string_construct_with_string(kr_string* string, unsigned block_size)
// with data-duplication.
{
  kr_string* new_string = (kr_string*)malloc(sizeof(kr_string));
	new_string->count = string->count;
	if (block_size) {
	  new_string->_block_size = block_size;
	} else {
	  new_string->_block_size = string->count * sizeof(kr_char);
	}
	new_string->_has_real_data = TRUE;
	new_string->_mem_size = ((string->count * sizeof(kr_char) - 1) / new_string->_block_size + 1) * new_string->_block_size;
	
	new_string->data = (kr_char*)malloc(new_string->_mem_size);
	memcpy(new_string->data, string->data, string->count * sizeof(kr_char));
	return new_string;
}

kr_string* kr_string_construct_with_data(kr_char* data, unsigned count, unsigned block_size)
// without data-duplication. directly set the data as Pointer.
{
  kr_string* new_string = (kr_string*)malloc(sizeof(kr_string));
	new_string->data = data;
	new_string->count = count;
  if (block_size) {
	  new_string->_block_size = block_size;
	} else {
	  new_string->_block_size = count * sizeof(kr_char);
	}
	new_string->_has_real_data = TRUE;
	new_string->_mem_size = ((count * sizeof(kr_char) - 1) / new_string->_block_size + 1) * new_string->_block_size;
	return new_string;
}

void kr_string_destruct(kr_string* target)
{
  if (target->_has_real_data){
	  free(target->data);
	}
  free(target);
}

void kr_string_print_to_stream(kr_string* target)
{
  printf("kr_string :\n count = 0x%x(%d)\n", target->count, target->count);
	printf(" _mem_size = 0x%X(%d)\n", target->_mem_size, target->_mem_size);
	printf(" _block_size = 0x%X(%d)\n", target->_block_size, target->_block_size);
  printf(" _has_real_data = %x\n", target->_has_real_data);
	unsigned i;
  for (i = 0; i < target->count; i++){
    printf("  %d : 0x%X\n", i, target->data[i]);
  }
}


kr_string* kr_string_substring(kr_string* target, kr_range* range)
{
  kr_string* string = (kr_string*)malloc(sizeof(kr_string));
	string->data = target->data + range->location;
	string->count = range->length;
	string->_mem_size = range->length * sizeof(kr_char);
	string->_has_real_data = FALSE;
	return string;
}

int kr_string_equal_with_length(kr_string* strA, kr_string* strB, unsigned length)
{
  if (strA->count > length || strB->count > length || strA->count != strB->count){ return FALSE; }
	unsigned i;
	for (i = 0; i < strA->count; i++){
	  if (strA->data[i] != strB->data[i]){ return FALSE; }
	}
	return TRUE;
}




kr_expr* kr_expr_construct_with_string(kr_string* string, kr_expr_type type, int mask)
{
  kr_expr* expr = (kr_expr*)malloc(sizeof(kr_expr));
	expr->expr_type = type;
	expr->repeat_min = expr->repeat_max = expr->repeat_current = 1;
	expr->is_matched_group = FALSE;
	expr->search_mask = mask;
	expr->expr_array = kr_array_construct(16, 16);
	expr->expr_string = string;
	
  unsigned i;
	kr_char chr;
	kr_range* range = kr_range_construct(0, 0);
	for (i = 0; i < string->count; i++){
	  chr = string->data[i];
		switch(chr){
      case KR_CHAR_PARENTHESIS_LEFT:
      {
        break;
      }
      case KR_CHAR_SQUARE_BLACKET_LEFT:
      {
        break;
      }
      case KR_CHAR_CURLY_BLACKET_LEFT:
      {
        break;
      }
      default:
      {
        
        break;
      }
    }
	}
	kr_range_destruct(range);
	return expr;
}

void kr_expr_destruct(kr_expr* expr)
{
  unsigned i;
	for (i = 0; i < expr->expr_array->count; i++){
	  kr_expr_destruct((kr_expr*)expr->expr_array->ptrs[i]);
	}
	kr_string_destruct(expr->expr_string);
  kr_array_destruct(expr->expr_array);
  free(expr);
}

void kr_expr_print_to_stream(kr_expr* expr)
{
  
}










