
#ifndef __kregex_h__
#define __kregex_h__

#define TRUE 1
#define FALSE 0
#define KR_RANGE_NOT_FOUND 0xFFFFFFFF
#define KR_REPEAT_TIME_MAX 0xFFFFFFFF

enum {
  KR_CHAR_SLASH = 0x2F, // '/'
	KR_CHAR_BACKSLASH = 0x5C, // '\'
	KR_CHAR_VERTICAL_BAR = 0x7C, // '|'
	
	KR_CHAR_SQUARE_BLACKET_LEFT = 0x5B,// '['
	KR_CHAR_SQUARE_BLACKET_RIGHT = 0x5D,// ']'
	KR_CHAR_CURLY_BLACKET_LEFT = 0x7B,// '{'
	KR_CHAR_CURLY_BLACKET_RIGHT = 0x7D,// '}'
	KR_CHAR_PARENTHESIS_LEFT = 0x28, // '('
	kR_CHAR_PARENTHESIS_RIGHT = 0x29, // ')'
	
	KR_CHAR_PLUS = 0x2B,// '+'
	KR_CHAR_HYPHEN = 0x2D,// '-'
	KR_CHAR_QUESTION = 0x3F,// '?'
	KR_CHAR_PERIOD = 0x2E,// '.'
	KR_CHAR_ASTERISK = 0x2A, // '*'
	KR_CHAR_DOLLAR = 0x24, // '$'
	KR_CHAR_SPACE = 0x20, // ' '
	KR_CHAR_CARET = 0x5E // '^'
};

enum {
  KR_SEARCH_IGNORE_CASE = 0x1,
	KR_SEARCH_MULTI_LINE = 0x2
};

typedef unsigned short kr_char;

typedef struct _kr_range {
  unsigned location;
  unsigned length;
} kr_range;

kr_range* kr_range_construct(unsigned location, unsigned length);
void kr_range_destruct(kr_range* range);
void kr_range_print_to_stream(kr_range* range);

kr_range* kr_range_duplicate(kr_range* range);
int kr_range_shift(kr_range* range, unsigned value); // if the proccess success, return TRUE; otherwise FALSE.


typedef struct _kr_array {
  unsigned* ptrs;
  unsigned count;
  unsigned _mem_size;
	unsigned _block_size;
} kr_array;

kr_array* kr_array_construct(unsigned count, unsigned block_size);
void kr_array_destruct(kr_array* array);
void kr_array_print_to_stream(kr_array* array);

void kr_array_add(kr_array* array, void* ptr); // without data-duplication.


typedef struct _kr_string {
  kr_char* data;
  unsigned count;
  unsigned _mem_size;
	unsigned _block_size;
	int _has_real_data;
} kr_string;

kr_string* kr_string_construct_with_string(kr_string* string, unsigned block_size); // with data-duplication.
kr_string* kr_string_construct_with_data(kr_char* data, unsigned count, unsigned block_size); // without data-duplication.

void kr_string_destruct(kr_string* target);
void kr_string_print_to_stream(kr_string* target);

kr_string* kr_string_substring(kr_string* target, kr_range* range); // without data-duplication.
int kr_string_equal_with_length(kr_string* strA, kr_string* strB, unsigned length);


typedef struct _kr_result {
  kr_range* range;
	kr_array* matched_ranges;
	unsigned repeat_min;
	unsigned repeat_current;
} kr_result;

kr_result* kr_result_construct();
void kr_result_destruct(kr_result* result);
void kr_result_print_to_stream(kr_result* result);



typedef enum {
  KR_EXPR_TYPE_GROUP, // expr_array : array of kr_expr* (multiple)
  KR_EXPR_TYPE_STRING, // expr_array : array of kr_string* (single)
	KR_EXPR_TYPE_WITHIN, // expr_array : array of kr_range* (multiple)
	KR_EXPR_TYPE_WITHOUT,  // expr_array : array of kr_range* (multiple)
	KR_EXPR_TYPE_OR,  // expr_array : array of kr_string* (multiple)
	KR_EXPR_TYPE_UNKNOWN // expr_array : unknown.
} kr_expr_type;


typedef struct _kr_expr {
	unsigned expr_type; // group, string, within, without, or
	unsigned repeat_min;
	unsigned repeat_max;
	unsigned repeat_current;
	int is_matched_group;
	int search_mask;
	
	kr_string* expr_string;
  kr_array* expr_array; 
} kr_expr;

kr_expr* kr_expr_construct_with_string(kr_string* string, kr_expr_type type, int mask);
void kr_expr_destruct(kr_expr* expr);
void kr_expr_print_to_stream(kr_expr* expr);

kr_result* kr_expr_match(kr_expr* expr, kr_string* string); // if no match, return 0.








#endif


